package org.exoplatform.frameworks.davclient.commands;

import java.util.ArrayList;

import org.apache.commons.logging.Log;
import org.exoplatform.frameworks.davclient.Const;
import org.exoplatform.frameworks.davclient.ServerLocation;
import org.exoplatform.frameworks.davclient.documents.DocumentApi;
import org.exoplatform.frameworks.davclient.documents.DocumentManager;
import org.exoplatform.services.log.ExoLogger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class DavPropFind extends DavCommand {
  
  private static Log log = ExoLogger.getLogger("jcr.DavPropFind");
  
  protected boolean isRequireAllProps = true;
  protected ArrayList<String> requiredProperties = new ArrayList<String>();
  
  protected DocumentApi multistatusDocument = null;
  
  public DavPropFind(ServerLocation location) throws Exception {
    super(location);
    commandName = Const.DavCommand.PROPFIND;
  }
  
  public void requireAllProperties() {
    isRequireAllProps = true;
  }
  
  public void setRequiredProperty(String propertyName) {
    isRequireAllProps = false;
    requiredProperties.add(propertyName);
  }
  
  public void finalExecute() {
    try {
      if (client.getReplyCode() == Const.HttpStatus.NOTFOUND) {
        return;
      }
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage(), exc);
    }
    multistatusDocument = DocumentManager.getResponseDocument(client.getResponseStream());
  }
  
  public DocumentApi getMultistatus() {
    return multistatusDocument; 
  }
  
  public Document toXml(Document xmlDocument) {
    Element propFindEl = xmlDocument.createElementNS(Const.Dav.NAMESPACE, 
        Const.Dav.PREFIX + Const.StreamDocs.PROPFIND);
    propFindEl.setAttribute(Const.Dav.NAMESPACEATTR, Const.Dav.NAMESPACE);
    xmlDocument.appendChild(propFindEl);
  
    if (isRequireAllProps) {
      Element allPropEl = xmlDocument.createElement(Const.Dav.PREFIX + Const.DavProp.ALLPROP);
      propFindEl.appendChild(allPropEl);      
    } else {
      Element propEl = xmlDocument.createElement(Const.Dav.PREFIX + Const.DavProp.PROP);
      propFindEl.appendChild(propEl);
      for (int i = 0; i < requiredProperties.size(); i++) {
        String curPropName = requiredProperties.get(i); 
        Element curPropEl = xmlDocument.createElement(Const.Dav.PREFIX + curPropName);
        propEl.appendChild(curPropEl);
      }
    }    
    
    return xmlDocument;
  }

  
}
