/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.dataflow;

import java.util.ArrayList;
import java.util.List;

import org.exoplatform.services.jcr.datamodel.InternalQPath;

/**
 * Created by The eXo Platform SARL        . <br/>
 * 
 * Stores collection of ItemState
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: ItemDataChangesLog.java 6078 2006-06-08 15:14:03Z peterit $
 * 
 */
public class ItemDataChangesLog {
  
  protected final List <ItemState> items;
  protected final String sessionId;
  
  public ItemDataChangesLog(List <ItemState> items, String sessionId) {
    this.items = items;
    this.sessionId = sessionId;
  }
  
  public ItemDataChangesLog(ItemState change, String sessionId) {
    this(new ArrayList <ItemState> (), sessionId);
    items.add(change);
  }

  /**
   * @return all states
   */
  public List <ItemState> getAllStates() {
    return items;
  }
  
  /**
   * @param id
   * @return current state (i.e. last change) of item data with UUID == id
   * or null if not found
   */
  public ItemState getItemState(String itemUuid) {
    for (ItemState state: getAllStates()) {
      if (state.getData().getUUID().equals(itemUuid)) {
        return state;
      }
    }
    return null;
  }
  
  /**
   * returns all states for with uuid == itemUuid
   * @param itemUuid
   * @return
   */
  public List <ItemState> getItemStates(String itemUuid) {
    List <ItemState> states = new ArrayList <ItemState>();
    for (ItemState state: getAllStates()) {
      if (state.getData().getUUID().equals(itemUuid)) {
        states.add(state);
      }
    }
    return states;
  }


  /**
   * @param qpath
   * @return current state (i.e. last change) of item data with qpath
   * or null if not found
   */
  public ItemState getItemState(InternalQPath itemPath) {
    for (ItemState state: getAllStates()) {
      if (state.getData().getQPath().equals(itemPath)) {
        return state;
      }
    }
    return null;
  }
  
  public String getSessionId() {
    return sessionId;
  }
  
  public String dump() {
    String str = "ChangesLog: \n";
    for(int i=0; i<items.size(); i++)
      str+=" "+items.get(i).getData().getQPath().getAsString()+" "+ItemState.getStateName(items.get(i).getState())+" \n";
    return str;
  }
}
