/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core.itemvisitors;

import java.util.Stack;

import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.dataflow.ItemState;
import org.exoplatform.services.jcr.datamodel.IllegalPathException;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.core.SessionDataManager;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeManagerImpl;
import org.exoplatform.services.jcr.impl.dataflow.TransientItemData;
import org.exoplatform.services.jcr.impl.dataflow.TransientNodeData;
import org.exoplatform.services.jcr.impl.dataflow.TransientPropertyData;
import org.exoplatform.services.jcr.impl.dataflow.session.SessionDataTraversingVisitor;
import org.exoplatform.services.jcr.impl.util.UUIDGenerator;
/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: $
 */

public class SessionCopyVisitor extends SessionDataTraversingVisitor {

  protected final InternalQPath ancestorToSave;
  
  protected final boolean keepUUIDs;

  protected final NodeTypeManagerImpl nodeTypeManager;
  
  private final Stack <NodeData> parents;
  
  private final InternalQName rootName;

  /**
   * @param dataManager
   * @param parentNode source node data
   * @param rootName destination node name
   */
  public SessionCopyVisitor(SessionDataManager dataManager, NodeData destParentNode, InternalQName destNodeName,
      boolean keepUUIDs, NodeTypeManagerImpl nodeTypeManager) {
    super(dataManager);
    this.parents = new Stack <NodeData>();
    this.ancestorToSave = destParentNode.getQPath();
    this.rootName = destNodeName;
    this.nodeTypeManager = nodeTypeManager;
    this.keepUUIDs = keepUUIDs;
  }

  @Override
  protected void entering(PropertyData property, int level)
      throws RepositoryException {
    dataManager.update(addedItemState(property, level), false);
  }

  @Override
  protected void entering(NodeData node, int level) throws RepositoryException {
    ItemState state = addedItemState(node, level);
    dataManager.update(state, false);
    parents.push((NodeData)state.getData());
  }

  @Override
  protected void leaving(PropertyData property, int level)
      throws RepositoryException {
  }

  @Override
  protected void leaving(NodeData node, int level) throws RepositoryException {
    parents.pop();
  }
  
  protected final ItemState addedItemState(ItemData data, int level) throws IllegalPathException {
    TransientItemData newData;
    InternalQName name = (level==0)?rootName:data.getQPath().getName();
    InternalQPath path = InternalQPath.makeChildPath(curParent().getQPath(), 
        name, data.getQPath().getIndex());
    
    String uuid;
    if(keepUUIDs && nodeTypeManager.isNodeType(Constants.MIX_REFERENCEABLE, ((NodeData)data).getMixinTypeNames()))
      uuid = data.getUUID();
    else
      uuid = UUIDGenerator.generate();
    
    if(data.isNode())
      newData = new TransientNodeData(
          path, uuid, -1,
          ((NodeData)data).getPrimaryTypeName(), ((NodeData)data).getMixinTypeNames(),
          ((NodeData)data).getOrderNumber(), curParent().getUUID(), 
          ((NodeData)data).getACL());
    else
      newData = new TransientPropertyData(path, uuid, -1,
          ((PropertyData)data).getType(), curParent().getUUID(), 
          ((PropertyData)data).isMultiValued());

      
    return new ItemState(newData, ItemState.ADDED, true, ancestorToSave); 
  }
  
  protected final NodeData curParent() {
    return parents.peek();
  }

}
