/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core.version;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import javax.jcr.ItemExistsException;
import javax.jcr.ItemNotFoundException;
import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.version.Version;
import javax.jcr.version.VersionHistory;

import org.exoplatform.services.jcr.core.ExtendedPropertyType;
import org.exoplatform.services.jcr.dataflow.DataManager;
import org.exoplatform.services.jcr.dataflow.ItemState;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.ValueData;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.PropertyImpl;
import org.exoplatform.services.jcr.impl.core.SessionDataManager;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.core.value.NameValue;
import org.exoplatform.services.jcr.impl.dataflow.MutableItemDataChangesLog;
import org.exoplatform.services.jcr.impl.dataflow.TransientNodeData;
import org.exoplatform.services.jcr.impl.dataflow.TransientPropertyData;
import org.exoplatform.services.jcr.impl.dataflow.TransientValueData;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov
 *         </a>
 * @version $Id: VersionImpl.java 8214 2006-08-23 12:12:55Z peterit $
 */

public class VersionImpl extends VersionStorageDescendantNode implements Version {
  

  public VersionImpl(NodeData data, SessionImpl session) throws PathNotFoundException,
      RepositoryException {

    super(data, session);
    
    if (!this.getPrimaryNodeType().isNodeType("nt:version"))
      throw new RepositoryException("Node " + getLocation().getAsString(true)
          + " is not nt:version type");
  }

  public VersionHistory getContainingHistory() throws RepositoryException {
    return (VersionHistory) getParent();
  }

  public Calendar getCreated() throws RepositoryException {
    return getProperty("jcr:created").getDate();
  }

  /* (non-Javadoc)
   * @see javax.jcr.version.Version#getSuccessors()
   */
  public Version[] getSuccessors() throws RepositoryException {
    Value[] refs = new Value[0];
    try {
      refs = getProperty("jcr:successors").getValues();
    } catch (PathNotFoundException e) {
    }
    Version[] successors = new Version[refs.length];
    for (int i = 0; i < refs.length; i++) {
      successors[i] = (Version) session.getNodeByUUID(refs[i].getString());
    }
    return successors;
  }

  /* (non-Javadoc)
   * @see javax.jcr.version.Version#getPredecessors()
   */
  public Version[] getPredecessors() throws RepositoryException {
    Value[] refs = new Value[0];
    try {
      refs = getProperty("jcr:predecessors").getValues();
    } catch (PathNotFoundException e) {
    }
    Version[] ps = new Version[refs.length];
    for (int i = 0; i < refs.length; i++) {
      ps[i] = (Version) session.getNodeByUUID(refs[i].getString());
    }
    return ps;
  }

  public void addSuccessor(Node successor) throws RepositoryException {
    PropertyImpl successors = null;
    try {
      successors = (PropertyImpl) getProperty("jcr:successors");
    } catch (PathNotFoundException e) {
    }
    Value successorRef = session.getValueFactory().createValue(successor);
    if (successors != null) {
      Value[] refs = successors.getValues();
      Value[] successorsRefs = new Value[refs.length + 1];
      for (int i = 0; i < refs.length; i++)
        successorsRefs[i] = refs[i];
      successorsRefs[refs.length] = successorRef;
      
      successors = createChildProperty("jcr:successors", successorsRefs, PropertyType.REFERENCE);
      dataManager.update(ItemState.createAdded(
          TransientPropertyData.createPropertyData(
              nodeData(), 
              Constants.EXO_PERMISSIONS, 
              ExtendedPropertyType.PERMISSION, 
              false,
              permValues)),
              true);
      
    } else {
      Value[] refs = new Value[] {successorRef};
      successors = createChildProperty("jcr:successors", refs, PropertyType.REFERENCE);
    }
    
    // [PN] 07.04.06
    session.getTransientNodesManager().getTransactManager().saveItem(successors);
    
  }

  /**
   * @param forSuccessors which property (true- successors, false-predeccessors)
   * @param uuid
   * @throws RepositoryException
   */
  void removeRefValue(boolean forSuccessors, String uuid) throws RepositoryException {
    String name = forSuccessors?"jcr:successors":"jcr:predecessors";
    if(!hasProperty(name))
      return;
    Value[] refs = getProperty(name).getValues();
    List newList = new ArrayList();
    for (int i = 0; i < refs.length; i++) {
      if(!refs[i].getString().equals(uuid))
        newList.add(refs[i]);
    }
    Value[] newRefs = new Value[newList.size()];
    for (int i = 0; i < newRefs.length; i++) {
      newRefs[i] = (Value)newList.get(i);
    }
    // TODO fix state ADDED to UPDATED inside true
    PropertyImpl prop = createChildProperty(name, newRefs, PropertyType.REFERENCE); 
    // [PN] 12.04.06
    session.getTransientNodesManager().getTransactManager().saveItem(prop);
  }

  
  public void addPredecessor(Version predecessor, MutableItemDataChangesLog changesLog) throws RepositoryException {
//    PropertyImpl predecessors = null;
//    try {
//      predecessors = (PropertyImpl) getProperty("jcr:predecessors");
//    } catch (PathNotFoundException e) {
//    }
//    Value predecessorRef = session.getValueFactory().createValue(predecessor);
//    if (predecessors != null) {
//      Value[] refs = predecessors.getValues();
//      Value[] successorsRefs = new Value[refs.length + 1];
//      for (int i = 0; i < refs.length; i++)
//        successorsRefs[i] = refs[i];
//      successorsRefs[refs.length] = predecessorRef;
//      predecessors = createChildProperty("jcr:predecessors", successorsRefs, PropertyType.REFERENCE);
//    } else {
//      Value[] refs = new Value[] { predecessorRef };
//      predecessors = createChildProperty("jcr:predecessors", refs, PropertyType.REFERENCE);
//    }

    //session.getTransientNodesManager().getTransactManager().saveItem(predecessors);
    
    PropertyData predecessors = (PropertyData) dataManager.getItemData(
        InternalQPath.makeChildPath(
            getLocation().getInternalPath(), 
            Constants.JCR_PREDECESSORS));
    
    List<ValueData> predRefs = predecessors.getValues();
    predRefs.add(new TransientValueData(predecessor.getUUID()));
    
    // TODO Will use existing PropertyData object, so it will be stored in ItemState etc. 
//    predecessors = TransientPropertyData.createPropertyData(
//        nodeData(), 
//        Constants.JCR_MERGEFAILED, 
//        PropertyType.REFERENCE, 
//        true, 
//        predRefs);
    changesLog.add(ItemState.createUpdated(predecessors));
  }
  
  public NodeImpl restore(NodeImpl node, boolean removeExisting) throws RepositoryException {

    NodeImpl frozenNode = (NodeImpl) getNode("jcr:frozenNode");

    SessionImpl nodeSession = (SessionImpl) node.getSession();
    SessionDataManager sdm = nodeSession.getTransientNodesManager();
    DataManager wdm = sdm.getTransactManager().getStorageDataManager();
    
    if (!removeExisting) {
      // [PN] 13.04.06 fast check first
      try {
        // same UUID node
        NodeImpl sameUUIDNode = (NodeImpl) nodeSession.getNodeByUUID(frozenNode.getProperty("jcr:frozenUuid").getString());
        if (!sameUUIDNode.getLocation().equals(node.getLocation())) {
          throw new ItemExistsException("Item with the same UUID as restored node "+node.getPath()
              + " already exists and removeExisting = false. Existed "+sameUUIDNode.getPath() + " " + sameUUIDNode.getInternalUUID());
        }
      } catch(ItemNotFoundException e) {
        // hasn't same uuid node
      }
    }
    
    // make new node from frozen

    // Manually reloading data to actual state,
    // force lazy loading of childs - they will be used in previous node
    // removing (inside WDM)
    node.setRefreshToPersistedVersion(true);
    List<NodeImpl> nodeChilds = node.loadChildNodes();
    node.loadChildProperties();
    node.setRefreshToPersistedVersion(false);

    // get an actual data of the target node and clean all childs in data,
    // we will load them in visitor regarding the frozen node childs
    TransientNodeData nodeData = (TransientNodeData) node.getActualItemData();
//    nodeData.getChildNodes().clear();
//    nodeData.getChildProperties().clear();

    NodeImpl nodeParent = (NodeImpl) node.getParent();
    InternalQName nodeName = nodeData.getQName();

    String ptNameStr = frozenNode.getProperty("jcr:frozenPrimaryType").getString();
    InternalQName ptName = ((NameValue) frozenNode.getProperty("jcr:frozenPrimaryType").getValue())
        .getQName();
    InternalQName[] mtNames = {};
    Value[] mtValues = null;
    if (frozenNode.hasProperty("jcr:frozenMixinTypes")) {
      mtValues = frozenNode.getProperty("jcr:frozenMixinTypes").getValues();
      mtNames = new InternalQName[mtValues.length];
      for (int i = 0; i < mtNames.length; i++)
        mtNames[i] = ((NameValue) mtValues[i]).getQName();
    }
    String uuid = null;
    if (frozenNode.hasProperty("jcr:frozenUuid")) {
      uuid = frozenNode.getProperty("jcr:frozenUuid").getString();
    }

    // [PN] 06.05.06 TODO It's same as node but with status DELETED. Fix it in the refactoring.
    NodeImpl removedNode = (NodeImpl) nodeParent.getNode(node.getName());
    
    NodeImpl restoredNode = nodeParent.createChildNodeInmemory(nodeName, ptName, mtNames, uuid,
        node.getIndex(), false, false, false);
    restoredNode.loadData(nodeData);

    // copy childs/properties with OnParentVersionAction.IGNORE to the restored node
    NodeCopyIgnoredVisitor copyIgnoredVisitor = new NodeCopyIgnoredVisitor(restoredNode, true, false);
    node.accept(copyIgnoredVisitor);

    try {
      // ----------- BEGIN SESSION BATCH ----------    
      wdm.beginSessionBatch(nodeSession);
      
      // visitor accept frozenNode traverse and save state in WorkspaceDataManager
      RestoreVisitor visitor = new RestoreVisitor(restoredNode, removeExisting);
      frozenNode.accept(visitor);
  
      restoredNode.createChildProperty("jcr:primaryType", new Value[] { session.getValueFactory()
          .createValue(ptNameStr, PropertyType.NAME) }, PropertyType.NAME);
      restoredNode.createChildProperty("jcr:uuid", new Value[] { session.getValueFactory()
          .createValue(uuid) }, PropertyType.REFERENCE);
      if (mtValues != null)
        restoredNode.createChildProperty("jcr:mixinTypes", mtValues, PropertyType.NAME);
  
      restoredNode.createChildProperty("jcr:baseVersion", new Value[] { session.getValueFactory()
          .createValue(this) }, PropertyType.REFERENCE);
      restoredNode.createChildProperty("jcr:isCheckedOut", new Value[] { session.getValueFactory()
          .createValue(false) }, PropertyType.BOOLEAN);
      
      sdm.getTransactManager().saveItem(removedNode); // delete
      // [PN] 04.07.06 For restored node we disabling validation (Caused by restore 'nt:file' error)
      sdm.getTransactManager().saveItem(restoredNode, false); // add
      
      wdm.endSessionBatch(nodeSession);
      //----------- END SESSION BATCH ----------
      
      sdm.resetItemState(restoredNode);
      
      sdm.reloadCache(restoredNode.getLocation(), true);
    
    } catch(ItemExistsException e) {
      // rollback some changes
      nodeParent.loadData(wdm.getItemData(nodeParent.getInternalUUID())); // rollback node's parent to stored state
      throw e;
    }
    return restoredNode;
  }
  
  
  public boolean isSuccessorOrSameOf(VersionImpl anotherVersion) 
  throws RepositoryException {
    Version[] prds = getPredecessors();
    for(int i=0; i<prds.length; i++) {
      if(prds[i].getUUID().equals(anotherVersion.getUUID()) ||
         ((VersionImpl)prds[i]).isSuccessorOrSameOf(anotherVersion))
        return true;
    }
    return false;
  }
 
}