/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.dataflow;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.jcr.PathNotFoundException;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.ValueFormatException;

import org.exoplatform.services.jcr.dataflow.DataManager;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.ValueData;
import org.exoplatform.services.jcr.impl.core.value.BaseValue;
import org.exoplatform.services.jcr.impl.core.value.StringValue;
import org.exoplatform.services.jcr.impl.core.value.ValueFactoryImpl;

/**
 * Created by The eXo Platform SARL 15.05.2006
 * 
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: PropertyDataReader.java 8214 2006-08-23 12:12:55Z peterit $
 */
public class PropertyDataReader extends ItemDataReader {

  private HashMap<String, PropertyInfo> propeties = new HashMap<String, PropertyInfo>();
  
  private class PropertyInfo {
    private InternalQName propertyName = null;
    private boolean multiValued = false;
    private List<Value> mValue = null;
    private Value value = null;
    private int type = PropertyType.STRING;

    PropertyInfo(InternalQName propertyName, int type) {
      this.propertyName = propertyName;
      this.type = type;
    }
    
    public InternalQName getPropertyName() {
      return propertyName;
    }

    public boolean isMultiValued() {
      return multiValued;
    }
    
    public void setMultiValued(boolean multiValued) {
      this.multiValued = multiValued;
    }
    
    public List<Value> getValues() throws ValueFormatException, PathNotFoundException {
      if (multiValued) {
        if (mValue != null) {
          return mValue;
        }
      } else if (value != null) {
        throw new ValueFormatException("Property " + parent.getQPath().getAsString()  
            + propertyName.getAsString() + " is multi-valued");
      }
      throw new PathNotFoundException("Property " + parent.getQPath().getAsString() 
          + propertyName.getAsString() + " not found (multi-valued)");
    }
    
    public void setValues(List<Value> mValue) {
      this.mValue = mValue;
      this.multiValued = true;
    }

    public Value getValue() throws ValueFormatException, PathNotFoundException {
      if (!multiValued) {
        if (value != null) {
          return value;
        }
      } else if (mValue != null) {
        throw new ValueFormatException("Property " + parent.getQPath().getAsString() 
            + propertyName.getAsString() + " is single-valued");
      }
      throw new PathNotFoundException("Property " + parent.getQPath().getAsString() 
          + propertyName.getAsString() + " not found (single-valued)");
    }
    
    public void setValue(Value value) {
      this.value = value;
      this.multiValued = false;
    }

    public int getType() {
      return type;
    }
  }
  
  public PropertyDataReader(NodeData parent, DataManager dataManager, ValueFactoryImpl valueFactory) {
    super(parent, dataManager, valueFactory);
  }
    
  public PropertyDataReader forProperty(InternalQName name, int type) {
    propeties.put(name.getAsString(), new PropertyInfo(name, type));
    return this;
  }
  
  public List<Value> getPropertyValues(InternalQName name) throws ValueFormatException, PathNotFoundException {
    return propeties.get(name.getAsString()).getValues();
  }
  
  public Value getPropertyValue(InternalQName name) throws ValueFormatException, PathNotFoundException {
    return propeties.get(name.getAsString()).getValue();
  }
  
  public void read() throws RepositoryException {
    List<PropertyData> ndProps = dataManager.getChildPropertiesData(parent);
    for (PropertyData prop : ndProps) {
      if (propeties.containsKey(prop.getQPath().getName().getAsString())) {
        PropertyInfo propInfo = propeties.get(prop.getQPath().getName().getAsString());
        List<ValueData> valueDataList = prop.getValues();
        if (prop.isMultiValued()) {
          List<Value> values = new ArrayList<Value>();
          for (ValueData vd : valueDataList) {
            values.add(makeValue(vd, propInfo.getType()));
          }
          propInfo.setValues(values);
        } else {
          if (valueDataList.size()>0)
            propInfo.setValue(makeValue(valueDataList.get(0), propInfo.getType()));
        }
      }
    }
  }

  private Value makeValue(ValueData valueData, int type) throws RepositoryException {
    if (valueFactory != null)
      return valueFactory.loadValue(valueData, type);
    try {
      return new StringValue(new String(valueData.getAsByteArray(), BaseValue.DEFAULT_ENCODING));
    } catch(UnsupportedEncodingException e) {
      return new StringValue(new String(valueData.getAsByteArray()));
    }
  }
  
}
