/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.dataflow;

/**
 * Created by The eXo Platform SARL        .
 * <br>
 * Newly added Node's data (used for mock inmemory repository as well). 
 * Besides NodeData's methods includes child items adders
 *
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: TransientNodeData.java 7556 2006-08-03 19:52:16Z geaz $
 */

import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.access.AccessControlList;
import org.exoplatform.services.jcr.dataflow.ItemDataVisitor;
import org.exoplatform.services.jcr.datamodel.IllegalPathException;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.MutableNodeData;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.impl.util.UUIDGenerator;

public class TransientNodeData extends TransientItemData implements Comparable,
    MutableNodeData  {
 
  private InternalQName primaryTypeName;

  private InternalQName[] mixinTypeNames;

  private int orderNum;

  private AccessControlList acl;
  
  public TransientNodeData(InternalQPath path, String uuid, int version,
      InternalQName primaryTypeName, InternalQName[] mixinTypeNames,
      int orderNum, String parentUUID, AccessControlList acl) {
    super(path, uuid, version, parentUUID);
    this.primaryTypeName = primaryTypeName;
    this.mixinTypeNames = mixinTypeNames;
    this.orderNum = orderNum;    
    this.acl = acl;
  }
  
  
  // --------------- ItemData ------------
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.ItemData#isNode()
   */
  public boolean isNode() {
    return true;
  }
  
  //---------------- NodeData -------------
  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.NodeData#getOrderNumber()
   */
  public int getOrderNumber() {
    return orderNum;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.NodeData#getPrimaryTypeName()
   */
  public InternalQName getPrimaryTypeName() {
    return primaryTypeName;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.NodeData#getMixinTypeNames()
   */
  public InternalQName[] getMixinTypeNames() {
    return mixinTypeNames;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.NodeData#getChildrenACL()
   */
  public AccessControlList getACL() {
    return acl;
  }
  
  //---------------- MutableNodeData 

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.MutableNodeData#setOrderNumber(int)
   */
  public void setOrderNumber(int orderNum) {
    this.orderNum = orderNum;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.MutableNodeData#setMixinTypeNames(org.exoplatform.services.jcr.datamodel.InternalQName[])
   */
  public void setMixinTypeNames(InternalQName[] mixinTypeNames) {
    this.mixinTypeNames = mixinTypeNames;
  }


  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.MutableNodeData#setUUID(java.lang.String)
   */
  public void setUUID(String uuid) {
    UUID = uuid;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.MutableNodeData#setACL(org.exoplatform.services.jcr.access.AccessControlList)
   */
  public void setACL(AccessControlList acl) {
    this.acl = acl;
  }

  public void accept(ItemDataVisitor visitor) throws RepositoryException {
    visitor.visit(this);
  }

  /**
   * Factory method
   * 
   * @param parent
   * @param name
   * @param primaryTypeName
   * @return
   */
  public static TransientNodeData createNodeData(NodeData parent,
      InternalQName name, InternalQName primaryTypeName) {
    TransientNodeData nodeData = null;
    try {
      InternalQPath path = InternalQPath.makeChildPath(parent.getQPath(), name.getAsString());
      nodeData = new TransientNodeData(path,
          UUIDGenerator.generate(), -1, primaryTypeName, 
          new InternalQName[0], 0, parent.getUUID(), new AccessControlList(true));

    } catch (IllegalPathException e) {
      // never thrown
    }
    return nodeData;
  }
    
  /**
   * Factory method
   * 
   * @param parent
   * @param name
   * @param primaryTypeName
   * @return
   */
  public static TransientNodeData createNodeData(NodeData parent,
      InternalQName name, InternalQName primaryTypeName, String uuid) {
    TransientNodeData nodeData = null;
    try {
      InternalQPath path = InternalQPath.makeChildPath(parent.getQPath(), name.getAsString());
      nodeData = new TransientNodeData(path,
          uuid, -1, primaryTypeName, 
          new InternalQName[0], 0, parent.getUUID(), new AccessControlList(true));

    } catch (IllegalPathException e) {
      // never thrown
    }
    return nodeData;
  }

  // ------------- Comparable  /////

  public int compareTo(Object obj) {
    return ((NodeData) obj).getOrderNumber() - orderNum;
  }
  
}

