/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.dataflow.persistent;

import java.util.List;

import javax.jcr.RepositoryException;
import javax.jcr.nodetype.NoSuchNodeTypeException;
import javax.jcr.nodetype.NodeType;

import org.exoplatform.services.jcr.dataflow.persistent.WorkspaceStorageCache;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeManagerImpl;
import org.exoplatform.services.jcr.impl.dataflow.data.ItemDataFactory;
import org.exoplatform.services.jcr.storage.WorkspaceDataContainer;

/**
 * Created by The eXo Platform SARL
 * Author : Peter Nedonosko
 *          peter.nedonosko@exoplatform.com.ua
 * 13.04.2006
 * @version $Id: CacheableWorkspaceDataManager.java 8214 2006-08-23 12:12:55Z peterit $
 */
public class CacheableWorkspaceDataManager extends WorkspaceDataManager {

  protected WorkspaceStorageCache cache = null;
  
  /**
   * @param dataContainer
   * @param cache
   */
  public CacheableWorkspaceDataManager(WorkspaceDataContainer dataContainer, WorkspaceStorageCacheImpl cache) {
    super(dataContainer);
    this.cache = cache;
    addItemPersistenceListener(cache);
  }  

  /**
   * 
   * @param nodeData
   * @param forcePersistentRead
   * @return
   * @throws RepositoryException
   */
  public List<NodeData> getChildNodesData(NodeData nodeData, boolean forcePersistentRead) throws RepositoryException {
    
    List<NodeData> childNodes = null;
    if (!forcePersistentRead && cache.isEnabled()) {
      childNodes = cache.getChildNodes(nodeData);
      if (childNodes != null) {
        //if (childNodes.size()<=0)
        //  log.info("CDM."+dataContainer.getName()+".getChildNodesData() CACHE '" + nodeData.getQPath().getAsString() + "' - " + childNodes.size());
        List<NodeData> copyOfNodes = ItemDataFactory.makeCopyOfNodes(childNodes);
        //if (copyOfNodes.size()<=0)
        //  log.info("CDM."+dataContainer.getName()+".getChildNodesData() CACHE COPY '" + nodeData.getQPath().getAsString() + "' - " + copyOfNodes.size());
        return copyOfNodes;
      }
    }
    
    childNodes = super.getChildNodesData(nodeData);
    if (cache.isEnabled()) {
      // [PN] getting parent from cache
      NodeData parentData = (NodeData) cache.get(nodeData.getUUID());
      if (parentData == null) {
        parentData = (NodeData) super.getItemData(nodeData.getUUID());
      }
      cache.addChildNodes(parentData, childNodes);
    }
    //if (childNodes == null || childNodes.size()<=0)
    //  log.info("CDM."+dataContainer.getName()+".getChildNodesData() STORAGE '" + nodeData.getQPath().getAsString() + "' - " + (childNodes != null ? childNodes.size() : "[null]"));
    List<NodeData> copyOfNodes = ItemDataFactory.makeCopyOfNodes(childNodes);
    //if (copyOfNodes == null || copyOfNodes.size()<=0)
    //  log.info("CDM."+dataContainer.getName()+".getChildNodesData() STORAGE COPY '" + nodeData.getQPath().getAsString() + "' - " + (copyOfNodes != null ? copyOfNodes.size() : "[null]"));
    return copyOfNodes;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.core.WorkspaceDataManager#getChildNodes(org.exoplatform.services.jcr.datamodel.NodeData)
   */
  @Override
  public List<NodeData> getChildNodesData(NodeData nodeData) throws RepositoryException {
    return getChildNodesData(nodeData, false);
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.core.WorkspaceDataManager#getChildProperties(org.exoplatform.services.jcr.datamodel.NodeData, boolean)
   */
  public List<PropertyData> getChildPropertiesData(NodeData nodeData, boolean forcePersistentRead) throws RepositoryException {
    
    List<PropertyData> childProperties = null;
    if (!forcePersistentRead && cache.isEnabled()) {
      childProperties = cache.getChildProperties(nodeData);
      if (childProperties != null) {
        return ItemDataFactory.makeCopyOfProperties(childProperties);
      }
    }
    
    //WorkspaceStorageConnection con = dataContainer.openConnection();
    childProperties = super.getChildPropertiesData(nodeData);
    if (cache.isEnabled()) {
      // [PN] getting parent from cache
      NodeData parentData = (NodeData) cache.get(nodeData.getUUID());
      if (parentData == null) {
        // TODO Retreive parent from storage. Does it needs.
        //parentData = (NodeData) con.getItemData(nodeData.getUUID());
        parentData = (NodeData) super.getItemData(nodeData.getUUID());
      }
      cache.addChildProperties(parentData, childProperties);
    }
    return ItemDataFactory.makeCopyOfProperties(childProperties);
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.core.WorkspaceDataManager#getChildProperties(org.exoplatform.services.jcr.datamodel.NodeData)
   */
  @Override
  public List<PropertyData> getChildPropertiesData(NodeData nodeData) throws RepositoryException {
    return getChildPropertiesData(nodeData, false);
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.core.WorkspaceDataManager#getChildNodesCount(org.exoplatform.services.jcr.datamodel.NodeData)
   */
  @Override
  public int getChildNodesCount(NodeData nodeData) throws RepositoryException {
    List<NodeData> childNodes = null;
    if (cache.isEnabled()) {
      childNodes = cache.getChildNodes(nodeData);
      if (childNodes != null) {
        return childNodes.size();
      }
    }
    return super.getChildNodesCount(nodeData);
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.core.WorkspaceDataManager#getChildPropertiesCount(org.exoplatform.services.jcr.datamodel.NodeData)
   */
  @Override
  public int getChildPropertiesCount(NodeData nodeData) throws RepositoryException {
    List<PropertyData> childProperties = null;
    if (cache.isEnabled()) {
      childProperties = cache.getChildProperties(nodeData);
      if (childProperties != null) {
        return childProperties.size();
      }
    }
    return super.getChildPropertiesCount(nodeData);
  }

  protected ItemData getCachedItemData(InternalQPath qpath) {
    return cache.get(qpath);
  }  
  
  protected ItemData getPersistedItemData(InternalQPath qpath) throws RepositoryException {

    ItemData data = null;
    //WorkspaceStorageConnection con = dataContainer.openConnection();
    //try {
    //  data = con.getItemData(qpath);
    //} finally {
    //  con.rollback();
    //}
    data = super.getItemData(qpath);
    if (data != null && cache.isEnabled()) {
      cache.put(data);
    }
    return data;
  }
  
  public ItemData getItemData(InternalQPath qpath) throws RepositoryException {
    // 2. Try from cache
    ItemData data = getCachedItemData(qpath);

    // 3. Try from container
    if (data == null) {
      data = getPersistedItemData(qpath);
    }
    return ItemDataFactory.makeItemData(data);
  }

  
  /** 
   * Returns an item from cache by UUID or null if the item don't cached.
   */  
  protected ItemData getCachedItemData(String uuid) {
    return cache.get(uuid);
  }

  /** 
   * Call {@link org.exoplatform.services.jcr.impl.dataflow.persistent.WorkspaceDataManager#getItemData(java.lang.String) WorkspaceDataManager.getItemDataByUUID(java.lang.String)}
   * and cache result if non null returned. 
   * @see org.exoplatform.services.jcr.impl.dataflow.persistent.WorkspaceDataManager#getItemData(java.lang.String)
   */
  //@Override
  protected ItemData getPersistedItemData(String uuid) throws RepositoryException {
    //ItemData data = super.getPersistedItemData(uuid);
    ItemData data = super.getItemData(uuid);
    if (data != null && cache.isEnabled()) {
      cache.put(data);
    }
    return data;
  }  
  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.core.WorkspaceDataManager#getItemDataByUUID(java.lang.String)
   */
  @Override
  public ItemData getItemData(String uuid) throws RepositoryException {
    // 2. Try from cache
    ItemData data = getCachedItemData(uuid);

    // 3. Try from container
    if (data == null) {
      data = getPersistedItemData(uuid);
    }
    return ItemDataFactory.makeItemData(data);
  }

  //-------------------- Custom NodeType detecting -----------------------
  
  /**
   * TODO Experimental.
   */
  private boolean isNodeTypeSimple(NodeData nodeData, InternalQName qName) {

    InternalQName primary = nodeData.getPrimaryTypeName();
    if (primary.equals(qName)) {
      return true;
    }
    
    InternalQName[] mixins = nodeData.getMixinTypeNames();
    for (InternalQName mixin: mixins) {
      if (mixin.equals(qName)) {
        return true;
      }
    }
    return false;
  }
}
