/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.xml;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;

import javax.jcr.NamespaceException;
import javax.jcr.NamespaceRegistry;
import javax.jcr.RepositoryException;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;

import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.util.NodeTypeRecognizer;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: NodeImporter.java 7373 2006-07-27 08:35:03Z geaz $
 */

public class NodeImporter implements ContentHandler, ErrorHandler {

  private ContentHandler wrappedHandler;

  protected NodeImpl     parent;

  protected int          uuidBehavior    = -1;

  private NamespaceRegistry nsReg;
  //private SessionImpl session;
  
  private PrintStream    userErrorStream = null;   
  
  private boolean saveOnEnd = false;

  public NodeImporter(NodeImpl parent) throws RepositoryException {
    this.parent = parent;
    this.nsReg = parent.getSession().getWorkspace().getNamespaceRegistry();
    //this.session = (SessionImpl) parent.getSession();
  }
  
  public NodeImpl parse(InputStream stream) throws IOException, SAXException,
      ParserConfigurationException, RepositoryException {

    // parse XML
    XMLReader reader;
    try {
      SAXParserFactory factory = SAXParserFactory.newInstance();
      factory.setNamespaceAware(true);
      reader = factory.newSAXParser().getXMLReader();
    } catch (Exception e) {
      throw new SAXException(e.getMessage());
    }
    reader.setContentHandler(this);
    reader.setErrorHandler(this);
    reader.parse(new InputSource(stream));

    // recognizeDocumentType(stream);

    if (wrappedHandler instanceof SysNodeImporter) {
      return ((SysNodeImporter) wrappedHandler).getNode();
    } else if (wrappedHandler instanceof DocNodeImporter) {
      return ((DocNodeImporter) wrappedHandler).getNode();
    } else {
      throw new SAXException("Handler is not set.");
    }
  }
  
  public void characters(char[] ch, int start, int length) throws SAXException {
    if (wrappedHandler == null)
      throw new SAXException("Handler is not set.");
    wrappedHandler.characters(ch, start, length);
  }

  public void endDocument() throws SAXException {
    if (wrappedHandler == null)
      throw new SAXException("Handler is not set.");
    wrappedHandler.endDocument();
    //session.setCacheEnabled(true);
    if (wrappedHandler instanceof DocNodeImporter) {
      if (isSaveOnEnd()) {
        try {
          // For save all imported nodes after import (from Workspace)
          ((DocNodeImporter) wrappedHandler).getNode().getSession().save();
        } catch(Exception e) {
          throw new SAXException(e.getMessage(), e);
        }
      }
    } else if (wrappedHandler instanceof SysNodeImporter) {
      // Clearing session node manager cache for sys view import only, experemental. Peter Nedonosko. 09.01.06
      //clearSessionCache();
      if (isSaveOnEnd()) {
        try {
          // [PN] 03.02.06 For save all imported nodes after import (from Workspace)
          ((SysNodeImporter) wrappedHandler).getNode().getSession().save();
        } catch(Exception e) {
          throw new SAXException(e.getMessage(), e);
        }
      }
    }
  }

  public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
    if (wrappedHandler == null)
      throw new SAXException("Handler is not set.");
    wrappedHandler.endElement(namespaceURI, localName, qName);
  }

  public void endPrefixMapping(String prefix) throws SAXException {
  }

  public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
  }

  public void processingInstruction(String target, String data) throws SAXException {
  }

  public void setDocumentLocator(Locator locator) {
  }

  public void skippedEntity(String name) throws SAXException {
  }

  public void startDocument() throws SAXException {
    
    //session.setCacheEnabled(false);
    //clearSessionCache();
  }

  public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
      throws SAXException {
    if (wrappedHandler == null) {
      try {
        recognizeDocumentType(namespaceURI, qName);
      } catch (RepositoryException e) {
        throw new SAXException(e);
      }
    }
    wrappedHandler.startElement(namespaceURI, localName, qName, atts);
  }

  public void startPrefixMapping(String prefix, String uri) throws SAXException {

    try {
      nsReg.getPrefix(uri);
    } catch (NamespaceException e) {
      try {
        nsReg.registerNamespace(prefix, uri);
      } catch (NamespaceException e1) {
        throw new RuntimeException(e1);
      } catch (RepositoryException e1) {
        throw new RuntimeException(e1);
      }

    } catch (RepositoryException e) {
      throw new RuntimeException(e);
    }
  }

  protected final void recognizeDocumentType(String namespaceURI, String qName)
      throws RepositoryException {

    if (NodeTypeRecognizer.recognize(namespaceURI, qName) == NodeTypeRecognizer.SYS) {
      wrappedHandler = new SysNodeImporter(parent);
    } else {
      wrappedHandler = new DocNodeImporter(parent, uuidBehavior);
    }
  }

  protected final void recognizeDocumentType(InputStream is) throws IOException, SAXException,
      ParserConfigurationException, RepositoryException {

    if (NodeTypeRecognizer.recognize(is) == NodeTypeRecognizer.SYS) {
      wrappedHandler = new SysNodeImporter(parent);
    } else {
      DocNodeImporter importer = new DocNodeImporter(parent, uuidBehavior);
      importer.setSaveOnEnd(isSaveOnEnd());
      wrappedHandler = importer;
    }
  }

  public void setUuidBehavior(int uuidBehavior) {
    this.uuidBehavior = uuidBehavior;
  }

  public int getUuidBehavior() {
    return uuidBehavior;
  }

  // ============ org.xml.sax.ErrorHandler implementation ============

  /*
   * (non-Javadoc)
   * 
   * @see org.xml.sax.ErrorHandler#error(org.xml.sax.SAXParseException)
   */
  public void error(SAXParseException err) throws SAXException {
    // TODO Auto-generated method stub
    errorErrorLog(null, err);    
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.xml.sax.ErrorHandler#fatalError(org.xml.sax.SAXParseException)
   */
  public void fatalError(SAXParseException err) throws SAXException {
    // TODO Auto-generated method stub
    fatalErrorLog(null, err);    
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.xml.sax.ErrorHandler#warning(org.xml.sax.SAXParseException)
   */
  public void warning(SAXParseException err) throws SAXException {
    // TODO Auto-generated method stub
    warnErrorLog(null, err);    
  }

  private void warnErrorLog(String message, Throwable exception) {
    toErrorLog("[WARN] " + (message != null && message.length() > 0 ? message + "; " : ""),
        exception);
  }

  private void errorErrorLog(String message, Throwable exception) {
    toErrorLog("[ERROR] " + (message != null && message.length() > 0 ? message + "; " : ""),
        exception);
  }

  private void fatalErrorLog(String message, Throwable exception) {
    toErrorLog("[FATAL] " + (message != null && message.length() > 0 ? message + "; " : ""),
        exception);
  }

  private void toErrorLog(String message, Throwable exception) {

    // very simple impl, Peter Nedonosko
    String err = "ErrorHandler (exo-jcr NodeImporter) " + (message != null && message.length() > 0 ? message + ": " : "")
        + "Exception: " + exception;
    PrintStream errStream = (userErrorStream != null ? userErrorStream : System.err);
    errStream.println(err);
    exception.printStackTrace(errStream);
  }

  public void setUserErrorStream(PrintStream printStream) {
    userErrorStream = printStream;
  }

  public PrintStream getUserErrorStream() {
    return userErrorStream;
  }

  public boolean isSaveOnEnd() {
    return saveOnEnd;
  }

  public void setSaveOnEnd(boolean saveOnEnd) {
    this.saveOnEnd = saveOnEnd;
  }
}
