package org.exoplatform.services.webdav;

import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.container.xml.ValueParam;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.webdav.DavConst.NodeTypes;

public class DavConfigImpl implements DavConfigApi {
  
  private static Log log = ExoLogger.getLogger("jcr.DavConfig");

  public static final String INIT_PARAM_DEFAULT_IDENTITY = "default-identity";
  public static final String INIT_PARAM_AUTH_HEADER = "auth-header";
  public static final String INIT_PARAM_AUTO_MIX_LOCKABLE = "auto-mix-lockable";
  public static final String INIT_PARAM_DEF_FOLDER_NODE_TYPE = "def-folder-node-type";
  public static final String INIT_PARAM_DEF_FILE_NODE_TYPE = "def-file-node-type";  
  
  protected String _defIdentity = "anonymous:anonymous";
  protected String _authHeader = "";
  protected boolean _autoMixLockable = false;
  protected String _defFolderNodeType = DavConst.NodeTypes.NT_FOLDER;
  protected String _defFileNodeType = DavConst.NodeTypes.NT_FILE;

  public DavConfigImpl(InitParams params) {
  
    ValueParam pIdentity = params.getValueParam(INIT_PARAM_DEFAULT_IDENTITY);
    if (pIdentity != null) {
      _defIdentity = pIdentity.getValue();
      log.info(INIT_PARAM_DEFAULT_IDENTITY + " = " + _defIdentity);
    }
    
    ValueParam pAuthHeader = params.getValueParam(INIT_PARAM_AUTH_HEADER);
    if (pAuthHeader != null) {
      _authHeader = pAuthHeader.getValue();
      log.info(INIT_PARAM_AUTH_HEADER + " = " + _authHeader);
    }
    
    ValueParam pAutoMixLocable = params.getValueParam(INIT_PARAM_AUTO_MIX_LOCKABLE);
    if (pAutoMixLocable != null) {
      _autoMixLockable = true;
      log.info(INIT_PARAM_AUTO_MIX_LOCKABLE + " = " + _autoMixLockable);
    }

    ValueParam pDefFolderNodeType = params.getValueParam(INIT_PARAM_DEF_FOLDER_NODE_TYPE);
    if (pDefFolderNodeType != null) {
      _defFolderNodeType = pDefFolderNodeType.getValue();
      log.info(INIT_PARAM_DEF_FOLDER_NODE_TYPE + " = " + _defFolderNodeType);
    } 
    
    ValueParam pDefFileNodeType = params.getValueParam(INIT_PARAM_DEF_FILE_NODE_TYPE);
    if (pDefFileNodeType != null) {
      _defFileNodeType = pDefFileNodeType.getValue();
      log.info(INIT_PARAM_DEF_FILE_NODE_TYPE + " = " + _defFileNodeType);
    }
    
  }
  
  public String getDefIdentity() {
    return _defIdentity;
  }
  
  public String getAuthHeader() {
    return _authHeader;
  }
  
  public boolean isAutoMixLockable() {
    return _autoMixLockable;
  }
  
  public String getDefFolderNodeType() {
    return _defFolderNodeType;
  }
  
  public String getDefFileNodeType() {
    return _defFileNodeType;
  }

}
