package org.exoplatform.services.webdav;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.command.impl.CommandService;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.webdav.commands.DavCommand;
import org.exoplatform.services.webdav.props.DisplayNameProp;
import org.exoplatform.services.webdav.props.PropApi;
import org.exoplatform.services.webdav.props.PropManager;
import org.exoplatform.services.webdav.request.PropFindDoc;
import org.exoplatform.services.webdav.request.RequestDocApi;
import org.exoplatform.services.webdav.structs.DMultistatus;
import org.exoplatform.services.webdav.structs.DResponse;
import org.exoplatform.services.webdav.structs.DavFileInfo;

public class DavServiceImpl implements DavServiceApi {
  
  private static Log log = ExoLogger.getLogger("jcr.DavServiceImpl");
  
  protected CommandService commandService = null;
  protected RepositoryService  repositoryService = null;
  protected DavSessionProvider sessionProvider = null;
  
  protected DavConfigApi config = null;
  
  public DavServiceImpl (
      InitParams params,
      CommandService commandService, 
      RepositoryService repositoryService) throws Exception {
    
    this.commandService = commandService;
    this.repositoryService = repositoryService;

    config = new DavConfigImpl(params);

    sessionProvider = new DavSessionProvider(this, repositoryService.getRepository());
  }

  public DavConfigApi getConfig() {
    return config;
  }

  // command service ---
  
  public DavCommand getCommand(String commandName) {    
    DavCommand command = (DavCommand)commandService.getCommand(commandName);     
    command.setDavService(this);
    return command;
  }
  
  public ArrayList<String> getAvailableCommands() {
    ArrayList<String> commands = new ArrayList<String>();    
    Iterator<String> commandIter = commandService.getNames();
    while (commandIter.hasNext()) {
      String curCommand = commandIter.next();
      commands.add(curCommand);
    }
    return commands;
  }
  
  //repository service ---
  
  public RepositoryService getRepositoryService() {
    return repositoryService;
  }
  
  public DavSessionProvider getSessionProvider() {
    return sessionProvider;
  }

  public InputStream getHtmlWorkspaces(String serverPrefix, String serverApp) {
    try {
      String []workspaces = repositoryService.getRepository().getWorkspaceNames(); 
      
      ArrayList<DavFileInfo> files = new ArrayList<DavFileInfo>();
      for (int i = 0; i < workspaces.length; i++) {
        DavFileInfo workspaceInfo = new DavFileInfo();
        workspaceInfo.setName(workspaces[i]);
        files.add(workspaceInfo);
      }
      
      HtmlContentManager htmlManager = new HtmlContentManager(serverPrefix, serverApp);
      return htmlManager.getHtmlStream("", files);      
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage());
      exc.printStackTrace();
    }
    return null;
  }
  
  public DMultistatus getPresetWorkspaces(String serverPrefix, RequestDocApi propsInfo) {
    if (!(propsInfo instanceof PropFindDoc)) {
      System.out.println("Class not equals PropFindDoc");
      return null;
    }
    
    PropFindDoc propFindDocument = (PropFindDoc)propsInfo;
    
    ArrayList<DResponse> responses = new ArrayList<DResponse>();
    String []workspaces = null;
    try {
      workspaces = repositoryService.getRepository().getWorkspaceNames();
    } catch (Exception exc) {
      log.info("Unhandled exception. " + exc.getMessage());
      exc.printStackTrace();
      return null;
    }
    
    for (String curWorkspace : workspaces) {
      DResponse response = new DResponse();
      response.setHref(serverPrefix + "/" + curWorkspace + "/");
      
      ArrayList<String> propNames = propFindDocument.getProperties();
      for (int i = 0; i < propNames.size(); i++) {
        String curPropertyName = propNames.get(i);

        PropApi davProp = PropManager.getPropertyByName(curPropertyName);
        
        while (true) {
          if (DavConst.DavProperty.DISPLAYNAME.equals(curPropertyName)) {
            ((DisplayNameProp)davProp).setDisplayName(curWorkspace);
            davProp.setStatus(DavConst.HttpStatus.OK);
            break;
          }
          if (DavConst.DavProperty.RESOURCETYPE.equals(curPropertyName)) {
            davProp.setStatus(DavConst.HttpStatus.OK);
            break;
          }
          break;
        }
        
        response.AddProperty(davProp);
        
      }
      
      responses.add(response);
    }
    
    return new DMultistatus(responses);
  }            
  
  
  
}
