package org.exoplatform.services.webdav;

import javax.jcr.Repository;

import javax.jcr.Credentials;
import javax.jcr.SimpleCredentials;
import javax.jcr.Session;
import javax.servlet.http.HttpServletRequest;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.logging.Log;
import org.exoplatform.services.log.ExoLogger;

public class DavSessionProvider {

  private static Log log = ExoLogger.getLogger("jcr.DavSessionProvider");
  
  private DavServiceImpl davService = null;
  private Repository repository = null;
  
  public DavSessionProvider(DavServiceImpl davService, Repository repository) {
    this.davService = davService;
    this.repository = repository;
  }  
    
  public DavSession getSession(String authHeader, String workspace) {
    Session session = null;
    
    Credentials credentials = null;    
    String decodedAuth = "";
    
    if (authHeader != null) {
      try {
        String []basic = authHeader.split(" ");
        if (basic.length >= 2 && basic[0].equalsIgnoreCase(HttpServletRequest.BASIC_AUTH)) {
          decodedAuth = new String(Base64.decodeBase64(basic[1].getBytes()));
        }        
      } catch (Exception exc) {
        log.info("Invalid authentication parameters. " + exc.getMessage());
      }
    } else {
      decodedAuth = davService.getConfig().getDefIdentity();
    }
    
    try {
      String []authParams = decodedAuth.split(":");
      credentials = new SimpleCredentials(authParams[0], authParams[1].toCharArray());
    } catch (Exception exc) {
      log.info("Can't apply authenticate parameters. " + exc.getMessage());
    }
    
    try {
      session = repository.login(credentials, workspace);
    } catch (Exception exc) {
      log.info("Can't login to workspace. " + exc.getMessage());
    }
    
    if (session == null) {
      return null;
    }
    
    return new DavSession(session);
  }

}
