package org.exoplatform.services.webdav;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.exoplatform.services.webdav.props.PropApi;
import org.exoplatform.services.webdav.structs.DMultistatus;
//import org.exoplatform.services.webdav.structs.props.DavProperty;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Document;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

public class DavUtil {
    
    public static Node getChildNode(Node node, String childName) {
      NodeList nodes = node.getChildNodes();
      for (int i = 0; i < nodes.getLength(); i++) {
        Node curNode = nodes.item(i);
        if (curNode.getLocalName() != null && curNode.getLocalName().equals(childName)) {
          return curNode;
        }
      }
      return null;
    }    
    
    public static Document GetDocumentFromInputStream(InputStream in) throws Exception {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setNamespaceAware(true);        
      DocumentBuilder builder = factory.newDocumentBuilder();
      Document document = builder.parse(in);
      return document;
    }
    
    public static Document GetDocumentFromRequest(HttpServletRequest request) throws Exception {
      Document document = null;
      String contentLength = request.getHeader(DavConst.Headers.CONTENTLENGTH);        
      if (contentLength != null && !contentLength.equals("0")) {
        document = GetDocumentFromInputStream(request.getInputStream());
      }
      return document;
    }    
    
    public static String GetStatusInfo(int status) {
      switch (status) {
        case DavConst.HttpStatus.OK:
          return DavConst.StatusDescr.OK;
        case DavConst.HttpStatus.NOTFOUND:
          return DavConst.StatusDescr.NOTFOUND;
        case DavConst.HttpStatus.CONFLICT:
          return DavConst.StatusDescr.CONFLICT;
      }
      return DavConst.StatusDescr.NOTFOUND;
    }

    public static void SendMultistatus(HttpServletResponse response, DMultistatus multistatus) throws Exception {
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      Document responseDoc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
      
      multistatus.toXml(responseDoc);
      
      OutputFormat format = new OutputFormat("xml", "UTF-8", true);
      XMLSerializer serializer = new XMLSerializer(out, format);
      serializer.setNamespaces(true);
      serializer.asDOMSerializer().serialize(responseDoc);
      
      byte []bytes = out.toByteArray();

      response.setStatus(DavConst.HttpStatus.MULTISTATUS);      
      response.addHeader(DavConst.Headers.CONTENTTYPE, "text/xml;charset=UTF-8 ");
      response.addHeader(DavConst.Headers.CONTENTLENGTH, String.format("%s", bytes.length));
      
      for (int i = 0; i < bytes.length; i++) {
        response.getOutputStream().write(bytes[i]);
        //System.out.print((char)bytes[i]);
      }
    }
    
    public static void sendSingleProperty(HttpServletResponse response, PropApi property) throws Exception {
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      Document responseDoc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
      
      Element propElement = responseDoc.createElementNS(DavConst.DAV_NAMESPACE, DavConst.DAV_PREFIX + DavConst.DavProperty.PROP);
      responseDoc.appendChild(propElement);
      
      property.toXml(responseDoc, propElement);
      
      OutputFormat format = new OutputFormat("xml", "UTF-8", true);
      XMLSerializer serializer = new XMLSerializer(out, format);
      serializer.setNamespaces(true);
      serializer.asDOMSerializer().serialize(responseDoc);
      
      byte []bytes = out.toByteArray();
      
      response.setStatus(DavConst.HttpStatus.OK);
      response.addHeader(DavConst.Headers.CONTENTTYPE, "text/xml;charset=UTF-8 ");
      response.addHeader(DavConst.Headers.CONTENTLENGTH, String.format("%s", bytes.length));
  
      //System.out.println("-------------------------------------------------");
      for (int i = 0; i < bytes.length; i++) {
        response.getOutputStream().write(bytes[i]);
        //System.out.print((char)bytes[i]);
      }
      //System.out.println("-------------------------------------------------");
    }
    
}
