package org.exoplatform.services.webdav;

import java.util.ArrayList;
import java.util.SortedMap;

import java.io.ByteArrayInputStream;
import java.io.InputStream;

import org.exoplatform.services.webdav.structs.DavFileInfo;

public class HtmlContentManager {
  
  public static final String FOLDER_IMAGE = "/images/folder.gif";
  public static final String FILE_IMAGE = "/images/file.gif";
  
  protected String serverPrefix;
  protected String serverApp;
  
  public HtmlContentManager(String serverPrefix, String serverApp) {
    this.serverPrefix = serverPrefix;
    this.serverApp = serverApp;
  }
  
  public InputStream getHtmlStream(String path, ArrayList<DavFileInfo>files) {
    String html = "<html>";    
    html += getHead();
    html += getBody(serverPrefix, path, files);
    html += "</html>";
    
    byte []dataBytes = new byte[html.length()];
    char []charArray = html.toCharArray();
    for (int i = 0; i < html.length(); i++) {
        dataBytes[i] = (byte)charArray[i];
    }
    return new ByteArrayInputStream(dataBytes);
  }
  
  protected String getHead() {
    String head = "<head>";
    
    head += "<title>eXo-Platform Web-DAV Server</title>";
    head += "<style>";
    head += "<!--H1 {font-family:Tahoma,Arial,sans-serif;color:#525D76;font-size:22px;}";
    head += "H3 {font-family:Tahoma,Arial,sans-serif;color:white;background-color:#525D76;font-size:14px;}";
    head += "BODY {font-family:Tahoma,Arial,sans-serif;color:black;background-color:white;}";
    head += "A {color : #4433FF; font-size:12px;}";
    head += "A.name {color : #4433FF; font-size:12px;}";
    head += "HR {color : #525D76;}-->";
    head += "</style>";
    
    head += "</head>";
    return head;
  }
  
  protected String getBody(String href, String path, ArrayList<DavFileInfo>files) {
    String body = "<body>";
    
    body += "<h1>eXo-Platform Web-DAV Server.</h1>";
    body += "<HR size='1' noshade='noshade'>";
    body += "<font color=blue size=1>";
    
    String tableContent = createFilesTable(href + path + "/", files);
    
    ArrayList<String> pathesArray = new ArrayList<String>();
    String []pathes = path.split("/");    
    for (int i = 0; i < pathes.length; i++) {
      if ("".equals(pathes[i])) {
        continue;
      }
      pathesArray.add(pathes[i]);
    }
    
    for (int i = pathesArray.size() - 1; i >= 0; i--) {
      String curPathName = pathesArray.get(i);
      String curHref = "";
      for (int j = 0; j < i; j++) {
        curHref += "/" + pathesArray.get(j);
      }
      tableContent = createFolderTable(href + curHref + "/" + curPathName, curPathName, tableContent);
    }
    
    tableContent = createFolderTable(href, "root", tableContent);
    
    body += tableContent;    
    body += "</font>";
    body += "<HR size='1' noshade='noshade'>";
    body += "<h3>* summary 6 files *</h3>";
    body += "</body>";
    return body;
  }

  protected ArrayList<DavFileInfo> sortList(ArrayList<DavFileInfo> files) {
    ArrayList<DavFileInfo> sorted = new ArrayList<DavFileInfo>();
    
    for (int i = 0; i < files.size(); i++) {
      DavFileInfo curFileInfo = files.get(i);

      if (sorted.size() == 0) {
        sorted.add(curFileInfo);
      } else {
        boolean inserted = false;
        for (int pi = 0; pi < sorted.size(); pi++) {
          DavFileInfo sortedInfo = sorted.get(pi);
          if (sortedInfo.getName().compareTo(curFileInfo.getName()) >= 0) {
            sorted.add(pi, curFileInfo);
            inserted = true;
            break;
          }
        }
        if (!inserted) {
          sorted.add(curFileInfo);
        }
      }
      
    }
    return sorted; 
  }
  
  protected ArrayList<DavFileInfo> sortDirectory(ArrayList<DavFileInfo>files) {
    ArrayList<DavFileInfo> unsfolders = new ArrayList<DavFileInfo>();
    ArrayList<DavFileInfo> unsfiles = new ArrayList<DavFileInfo>();

    for (int i = 0; i < files.size(); i++) {
      DavFileInfo curFileInfo = files.get(i);
      if (curFileInfo.isCollection()) {
        unsfolders.add(curFileInfo);
      } else {
        unsfiles.add(curFileInfo);
      }
    }
    
    ArrayList<DavFileInfo> sortFolders = sortList(unsfolders);
    ArrayList<DavFileInfo> sortFiles = sortList(unsfiles);
    
    ArrayList<DavFileInfo> sortedAll = new ArrayList<DavFileInfo>();
    
    for (int i = 0; i < sortFolders.size(); i++) {
      DavFileInfo curFileInfo = sortFolders.get(i);
      sortedAll.add(curFileInfo);
    }

    for (int i = 0; i < sortFiles.size(); i++) {
      DavFileInfo curFileInfo = sortFiles.get(i);
      sortedAll.add(curFileInfo);
    }
    
    return sortedAll;
  }
  
  protected String createFilesTable(String href, ArrayList<DavFileInfo> files) {
    ArrayList<DavFileInfo> sorted = sortDirectory(files);
    
    String table = "<table border=0 width=100%>";

    for (int i = 0; i < sorted.size(); i++) {      
      DavFileInfo curFileInfo = sorted.get(i); 
      
      String fileName = curFileInfo.getName();
      String fileHref = href + fileName;
      
      String fileSize = "";
      if (!curFileInfo.isCollection()) {
        fileSize = String.format("%d", curFileInfo.getContentLength());
      }
      
      String contentType = curFileInfo.getContentType();
      if (contentType == null) {
        contentType = "&nbsp;";
      }
      
      String lastModified = curFileInfo.getLastModified();
      if (lastModified == null) {
        lastModified = "&nbsp;";
      }
      
      String image = serverApp + ((curFileInfo.isCollection()) ? FOLDER_IMAGE : FILE_IMAGE); 
      
      table += "<tr>";
      table += "<td width=10><img src=" + image + " width=17></td>";
      table += "<td><b><a href='" + fileHref + "'>" + fileName + "</a></b></td>";
      table += "<td><font size=2>" + fileSize + "</font></td>";
      table += "<td><font size=2>" + contentType + "</font></td>";
      table += "<td><font size=2 nowrap>" + lastModified + "</font></td>";
      table += "<td width=40%>&nbsp;</td>";
      table += "</tr>";            
    }
    
    table += "</table>";
    
    return table;
  }
  
  protected String createFolderTable(String href, String name, String tableContent) {
    String table = "<table border=0 width=100%>";     
    table += "<tr>";
    table += "<td><img src='" + serverApp + FOLDER_IMAGE + "' width=17></td>";
    table += "<td nowrap><b><a href='" + href + "'>" + name + "</a></b></td>";
    table += "<td width=100%>&nbsp;</td>";
    table += "</tr>";
    table += "<tr>";
    table += "<td>&nbsp;</td>";
    table += "<td colspan=2>" + tableContent + "</td>";
    table += "</tr></table>";
    return table;
  }
    
}
