package org.exoplatform.services.webdav.commands;

import org.apache.commons.chain.Command;
import org.apache.commons.chain.Context;  
import org.exoplatform.services.webdav.DavConst;
import org.exoplatform.services.webdav.DavContext;
import org.exoplatform.services.webdav.DavServiceImpl;
import org.exoplatform.services.webdav.DavSession;

public abstract class DavCommand implements Command {

  protected boolean needSrcSession = true;
  protected DavSession srcSession;
  protected boolean needDestSession = false;
  protected DavSession destSession;
  
  protected boolean status = false;
  protected DavServiceImpl davService = null;
  protected DavContext davContext = null;
  
  public void setDavService(DavServiceImpl davService) {
    this.davService = davService;
  }
  
  public final boolean execute(Context context) throws Exception {
    this.davContext = (DavContext)context;
    
    String defAuthHeader = davService.getConfig().getAuthHeader();
    
    if (needSrcSession) {
      srcSession = davContext.getSourceSession(davService.getSessionProvider(), davService.getConfig());
      if (srcSession == null) {
        System.out.println("Can't login to source workspace.");
        davContext.answerAuthNeeded(defAuthHeader);
        return false;
      }
    }
    
    if (needDestSession) {      
      if (!davContext.isSameHosts()) {
        System.out.println("Hosts are not equals.");
        srcSession.close();
        davContext.setResponseHeader(DavConst.Headers.WWWAUTHENTICATE, defAuthHeader);
        davContext.answerForbidden();
        return false;
      }
      
      destSession = davContext.getDestinationSession(davService.getSessionProvider());
      if (destSession == null) {
        System.out.println("Can't login to destination workspace.");
        srcSession.close();
        davContext.answerAuthNeeded(defAuthHeader);
        return false;
      }
    }
    
    status = process();
    
    if (srcSession != null) {
      srcSession.close();
    }
    if (destSession != null) {
      destSession.close();
    }
    
    return status;
  }
    
  protected abstract boolean process();
  


  
}
