package org.exoplatform.services.webdav.props;

import javax.jcr.Node;

import org.apache.commons.logging.Log;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.webdav.DavConst;
import org.exoplatform.services.webdav.DavConst.HttpStatus;
import org.exoplatform.services.webdav.DavConst.Lock;
import org.exoplatform.services.webdav.DavConst.NodeTypes;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class LockDiscoveryProp extends CommonProp {
  
  private static Log log = ExoLogger.getLogger("jcr.LockDiscoveryProp");
  
  protected boolean isLocked = false; 
  protected String lockType = DavConst.Lock.TYPE_WRITE;
  protected String lockScope = DavConst.Lock.SCOPE_EXCLUSIVE;
  protected int depth = -1;
  protected String timeOut = "";
  protected String owner = "eXo-Platform manage system.";
  protected String lockToken = "";
  
  public LockDiscoveryProp() {
    super(DavConst.DavProperty.LOCKDISCOVERY);
  }
  
  public void setLocked(boolean isLocked) {
    this.isLocked = isLocked;
  }
  
  public boolean getLocked() {
    return isLocked;
  }
  
  public void setLockType(String lockType) {
    this.lockType = lockType;
  }
  
  public String getLockType() {
    return lockType;
  }
  
  public void setLockScope(String lockScope) {
    this.lockScope = lockScope;
  }
  
  public String getLockScope() {
    return lockScope;
  }
  
  public void setDepth(int depth) {
    this.depth = depth;
  }
  
  public int getDepth() {
    return depth;
  }
  
  public void setTimeOut(String timeOut) {
    this.timeOut = timeOut;
  }
  
  public String getTimeOut() {
    return timeOut;
  }
  
  public void setOwner(String owner) {
    this.owner = owner;
  }
  
  public String getOwner() {
    return owner;
  }
  
  public void setLockToken(String lockToken) {
    this.lockToken = lockToken;
  }
  
  public String getLockToken() {
    return lockToken;
  }
  
  @Override
  public boolean fillFromNode(Node node, String href) {
    try {
      if (node.isLocked()) {
        isLocked = true;
        owner = node.getProperty(DavConst.NodeTypes.JCR_LOCKOWNER).getString();
      }
      status = DavConst.HttpStatus.OK;
      return true;
    } catch (Exception exc) {
      log.info("Can't get lockdiscovery for node. " + exc.getMessage());
      exc.printStackTrace();
    }
    return false;
  }  
  
  @Override
  public void toXml(Document rootDoc, Element parentElement) {
    super.toXml(rootDoc, parentElement);
    if (status != DavConst.HttpStatus.OK || !isLocked) {
      return;
    }
    
    Element elActiveLock = rootDoc.createElement(DavConst.DAV_PREFIX + DavConst.DavProperty.ACTIVELOCK);
    propertyElement.appendChild(elActiveLock);
    
    Element elLockType = rootDoc.createElement(DavConst.DAV_PREFIX + DavConst.DavProperty.LOCKTYPE);
    elActiveLock.appendChild(elLockType);
    Element elLockTypeWrite = rootDoc.createElement(DavConst.DAV_PREFIX + DavConst.DavProperty.WRITE);
    elLockType.appendChild(elLockTypeWrite);
    
    Element elLockScope = rootDoc.createElement(DavConst.DAV_PREFIX + DavConst.DavProperty.LOCKSCOPE);
    elActiveLock.appendChild(elLockScope);
    Element elLockScopeExclusicve = rootDoc.createElement(DavConst.DAV_PREFIX + DavConst.DavProperty.EXCLUSIVE);
    elLockScope.appendChild(elLockScopeExclusicve);
    
    Element elDepth = rootDoc.createElement(DavConst.DAV_PREFIX + DavConst.DavProperty.DEPTH);
    elActiveLock.appendChild(elDepth);
    if (depth < 0) {
      elDepth.setTextContent("Infinity");
    } else {
      elDepth.setTextContent(String.format("%s", depth));
    }
    
    Element elOwner = rootDoc.createElement(DavConst.DAV_PREFIX + DavConst.DavProperty.OWNER);
    elActiveLock.appendChild(elOwner);
    elOwner.setTextContent(owner);
    
    Element elTimeOut = rootDoc.createElement(DavConst.DAV_PREFIX + DavConst.DavProperty.TIMEOUT);
    elActiveLock.appendChild(elTimeOut);
    if ("".equals(timeOut)) {
      elTimeOut.setTextContent("Infinity");
    } else {
      elTimeOut.setTextContent(timeOut);
    }

    if (!"".equals(lockToken)) {
      Element elLockToken = rootDoc.createElement(DavConst.DAV_PREFIX + DavConst.DavProperty.LOCKTOKEN);
      elActiveLock.appendChild(elLockToken);
      Element elLockTokenHref = rootDoc.createElement(DavConst.DAV_PREFIX + DavConst.DavProperty.HREF);
      elLockToken.appendChild(elLockTokenHref);
      elLockTokenHref.setTextContent(lockToken);
    }
    
  }

}
