package org.exoplatform.services.webdav.props;

import java.util.ArrayList;

import javax.jcr.Node;
import javax.jcr.version.Version;

import org.apache.commons.logging.Log;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.webdav.DavConst;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class SuccessorSetProp extends CommonProp {

  private static Log log = ExoLogger.getLogger("jcr.SuccessorSetProp");
  
  protected ArrayList<String> versionSuccessors = new ArrayList<String>();
  
  public SuccessorSetProp() {
    super(DavConst.DavProperty.SUCCESSORSET);
  }
  
  @Override
  public boolean fillFromNode(Node node, String href) {
    try {
      if (!node.isNodeType(DavConst.NodeTypes.NT_VERSION)) {
        return false;
      }
      if (!(node instanceof Version)) {
        return false;
      }
      
      Version []successors = ((Version)node).getSuccessors();
      while (successors.length > 0) {
        versionSuccessors.add(href + DavConst.DAV_VERSIONPREFIX + successors[0].getName());
        successors = successors[0].getSuccessors();
      }
      
      status = DavConst.HttpStatus.OK;      
      return true;
    } catch (Exception exc) {
      log.info("Can't get successors. " + exc.getMessage());
      exc.printStackTrace();
    }
    return false;
  }  
  
  @Override
  public void toXml(Document rootDoc, Element parentElement) {
    super.toXml(rootDoc, parentElement);
    if (status != DavConst.HttpStatus.OK) {
      return;
    }
    for (int i = 0; i < versionSuccessors.size(); i++) {
      Element hrefEl = rootDoc.createElement(DavConst.DAV_PREFIX + DavConst.DavProperty.HREF);
      propertyElement.appendChild(hrefEl);
      hrefEl.setTextContent(versionSuccessors.get(i));
    }
  }
  
  
}
