package org.exoplatform.services.webdav.request;

import java.util.ArrayList;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.apache.commons.logging.Log;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.webdav.DavConst;
import org.exoplatform.services.webdav.DavUtil;
import org.exoplatform.services.webdav.DavConst.DavProperty;

public class PropFindDoc extends CommonPropDoc {

  private static Log log = ExoLogger.getLogger("jcr.PropFindDoc");
  
  protected static final String []defaultProperties = new String [] {
    DavConst.DavProperty.DISPLAYNAME,
    DavConst.DavProperty.RESOURCETYPE,
    DavConst.DavProperty.GETCONTENTTYPE,
    DavConst.DavProperty.GETCONTENTLENGTH,
    DavConst.DavProperty.GETLASTMODIFIED,
    DavConst.DavProperty.LOCKDISCOVERY,
    DavConst.DavProperty.CHECKEDIN,
    DavConst.DavProperty.CHECKEDOUT,
    DavConst.DavProperty.VERSIONNAME
  };
  
  public PropFindDoc() {
  }
  
  @Override
  public boolean initFromDocument(Document requestDocument) {
    try {
      Node propFind = DavUtil.getChildNode(requestDocument, DavConst.DavDocument.PROPFIND);
      Node propAllProp = DavUtil.getChildNode(propFind, DavConst.DavProperty.ALLPROP);
      if (propAllProp != null) {
        initWithDefaultProperties();
        return true;
      }
      
      Node props = DavUtil.getChildNode(propFind, DavConst.DavProperty.PROP);
      NodeList nodes = props.getChildNodes();
      for (int i = 0; i < nodes.getLength(); i++) {
        Node curNode = nodes.item(i);
        String localName = curNode.getLocalName();
        
        if (localName != null && DavConst.DAV_NAMESPACE.equals(curNode.getNamespaceURI())) {
          addProperty(curNode.getLocalName());
        }
        
      }

      Node propInclude = DavUtil.getChildNode(propFind, DavConst.DavProperty.INCLUDE);
      if (propInclude != null) {
        log.info("Needed extended include property.");
        // fill abseneded nodes.
      }
      
      return true;
    } catch (Exception exc) {
      log.info("Can't fill document data. " + exc.getMessage());
      exc.printStackTrace();      
    }
    return false;    
  }

  public void initWithDefaultProperties() {
    for (int i = 0; i < defaultProperties.length; i++) {
      addProperty(defaultProperties[i]);
    }
  }
  
}
