package org.exoplatform.services.webdav.request;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.webdav.DavConst;
import org.exoplatform.services.webdav.DavUtil;

public class RequestDocManager {
  
  private static Log log = ExoLogger.getLogger("jcr.RequestDocManager");
  
  protected static String [][]availableDocuments = {
    {DavConst.DavDocument.ACLPRINCIPALPROPS,
        AclPrincipalPropsDoc.class.getCanonicalName()},
    {DavConst.DavDocument.ACLPRINCIPALPROPSET,
        AclPrincipalPropSetDoc.class.getCanonicalName()},
    {DavConst.DavDocument.EXPANDPROPERTY,
        ExpandPropertyDoc.class.getCanonicalName()},
    {DavConst.DavDocument.LOCKINFO,
        LockInfoDoc.class.getCanonicalName()},
    {DavConst.DavDocument.PROPERTYBEHAVIOR,
        PropertyBehaviorDoc.class.getCanonicalName()},
    {DavConst.DavDocument.PROPERTYUPDATE,
        PropertyUpdateDoc.class.getCanonicalName()},
    {DavConst.DavDocument.PROPFIND,
        PropFindDoc.class.getCanonicalName()},
    {DavConst.DavDocument.VERSIONTREE,
        VersionTreeDoc.class.getCanonicalName()}
  };
  
  protected static RequestDocApi getDocumentClass(String documentName) throws Exception {
    for (int i = 0; i < availableDocuments.length; i++) {
      if (documentName.equals(availableDocuments[i][0])) {
        //log.info("Finded document - " + availableDocuments[i][0]);
        RequestDocApi doc = (RequestDocApi)Class.forName(availableDocuments[i][1]).newInstance();
        return doc;        
      }
    }    
    return null;
  }
  
  public static RequestDocApi getDocumentFromRequest(HttpServletRequest request) {
    try {
      Document doc = DavUtil.GetDocumentFromRequest(request);
      if (doc == null) {        
        if ("PROPFIND".equals(request.getMethod())) {
          PropFindDoc propsDoc = new PropFindDoc();
          propsDoc.initWithDefaultProperties();
          return propsDoc;
        }
        return null;
      }
      
      Node documentNode = doc.getChildNodes().item(0);
      if (!DavConst.DAV_NAMESPACE.equals(documentNode.getNamespaceURI())) {
        log.info("Input document namespace not equals DAV:");
        return null;
      }
      
      String documentName = documentNode.getLocalName();
      RequestDocApi requestDocument = getDocumentClass(documentName);
      if (requestDocument == null) {
        return null;
      }
      if (!requestDocument.initFromDocument(doc)) {
        return null;
      }
      return requestDocument;
    } catch (Exception exc) {
      log.info("Can't get document from request. " + exc.getMessage());
      exc.printStackTrace();
    }
    
    return null;
  }
  
}
