/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core.itemvisitors;

import java.util.Stack;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.util.TraversingItemVisitor;

import org.apache.commons.logging.Log;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.PropertyImpl;
import org.exoplatform.services.jcr.impl.dataflow.TransientNodeData;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL . Copies accepted node to parent structure
 * in memory
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov
 *         </a>
 * @version $Id: NodeCopyVisitor.java 9036 2006-09-21 15:53:50Z peterit $
 */

public class NodeCopyVisitor extends TraversingItemVisitor {

  protected static Log log = ExoLogger.getLogger("jcr.NodeCopyVisitor");
  
  protected Stack    parents;

  protected NodeImpl context;

  protected boolean  keepUUIDs;

  private boolean    registerInDM;

  public NodeCopyVisitor(NodeImpl context, boolean keepUUIDs, boolean registerInDataManager) {
    super();
    this.context = context;
    this.parents = new Stack();
    parents.push(context);
    this.keepUUIDs = keepUUIDs;
    this.registerInDM = registerInDataManager;
  }
  
  protected void entering(Property property, int level) throws RepositoryException {

    PropertyImpl propertyImpl = (PropertyImpl) property;
    Value[] values = propertyImpl.getValueArray();
    int type = property.getType();
    InternalQName qname = propertyImpl.getLocation().getInternalPath().getName();
    PropertyImpl newProperty = curParent().createChildPropertyInmemory(
        qname, type, values, propertyImpl.isMultiValued(), registerInDM, false);
    
    // [PN] 21.09.06 Fix jcr:uuid with actual value
    if (curParent().isNodeType(Constants.MIX_REFERENCEABLE) && qname.equals(Constants.JCR_UUID)) {
      Value[] vals = new Value[] {
          curParent().getSession().getValueFactory().createValue(curParent().getUUID())
          }; 
      newProperty.doSetValue(vals, false, type, newProperty.getState(), registerInDM, false);
    }      

    if (log.isDebugEnabled())
      log.debug("Copy property: " + propertyImpl.getPath() + " --> " + newProperty.getPath());
  }

  protected void entering(Node node, int level) throws RepositoryException {
    if (level == 0)
      return;

    NodeImpl nodeImpl = (NodeImpl) node;

    // [PN] 03.04.06
    TransientNodeData nodeData = (TransientNodeData) nodeImpl.getActualItemData();
    NodeImpl newNode = curParent().copyChildNodeInmemory(nodeData.getQName(),
        nodeData, keepUUIDs, true, registerInDM, false, false); // [PN] 15.04.06 addAutoCreated=false may be use true
    
    if (log.isDebugEnabled())
      log.debug("Copy node: " + nodeImpl.getPath() + " --> " + newNode.getPath());
    parents.push(newNode);
  }

  protected void leaving(Property property, int level) throws RepositoryException {
  }

  protected void leaving(Node node, int level) throws RepositoryException {
    parents.pop();
  }

  protected NodeImpl curParent() {
    return (NodeImpl) parents.peek();
  }

}
