/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.core.value;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import org.apache.commons.logging.Log;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL
 * Author : Peter Nedonosko
 *          peter.nedonosko@exoplatform.com.ua
 * 23.08.2006
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: DateFormatHelper.java 7757 2006-08-23 15:40:14Z peterit $
 */
public class DateFormatHelper {
  
  protected static Log log = ExoLogger.getLogger("jcr.DateFormatHelper");
  
  protected final DateFormat DATETZ_FORMAT = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ");
  protected final String CALENDAR_FIELDS_DELIMITER = ";"; // [PN] I hope it's unique will be for any time zone ID etc.
  protected final String CALENDAR_FIELDS_SEPARATOR = "--";
  
  Calendar deserialyze(String serString) {
    //System.out.println("Deserialyze date string: [" + serString + "] " + hashCode());
    String[] parts = serString.split(CALENDAR_FIELDS_SEPARATOR);
    if (parts.length == 2) {
      try {
        //Calendar isoDate = ISO8601.parse(parts[0]);
        //System.out.println("Parse Calendar date: [" + parts[0] + "] " + hashCode());
        
        // 22.09.06 [PN] imported from ISO8601.java
        // check optional leading sign (ERA)
        char sign;
        int start;
        if (parts[0].startsWith("-")) {
            sign = '-';
            start = 1;
        } else if (parts[0].startsWith("+")) {
            sign = '+';
            start = 1;
        } else {
            sign = '+'; // no sign specified, implied '+'
            start = 0;
        }
        
        Date isoDate = null;
      	try {
            isoDate = DATETZ_FORMAT.parse(parts[0].substring(start));
      	}catch (NumberFormatException e){
      	  //System.out.println("==Error:" + e + " [" + parts[0] + "] " + hashCode());
      	  throw e;
      	}
        
        Calendar isoCalendar = Calendar.getInstance();
        isoCalendar.setTime(isoDate);
        //System.out.println("Parse Calendar fields: [" + parts[1] + "] " + hashCode());
        String[] calendarFileds = parts[1].split(CALENDAR_FIELDS_DELIMITER);
        if (calendarFileds.length == 4) {
          try {
            isoCalendar.setLenient(Boolean.parseBoolean(calendarFileds[0]));
            isoCalendar.setFirstDayOfWeek(Integer.parseInt(calendarFileds[1]));
            isoCalendar.setMinimalDaysInFirstWeek(Integer.parseInt(calendarFileds[2]));
          } catch(Exception e) {
            log.warn("Can't parse serialized fields for the calendar '" 
                + parts[1] + "' but calendar has '" + isoDate.toString() + "'", e);
          }
        } else {
          log.warn("Fields of the calendar is not serialized properly '" 
              + parts[1] + "' but calendar has '" + isoDate.toString() + "'");
        }
        
        // 22.09.06 [PN] imported from ISO8601.java
        // Fix ERA
        //if (sign == '-' || year == 0) {
        if (sign == '-' || isoCalendar.get(Calendar.YEAR) == 0) {
          // not CE, need to set era (BCE) and adjust year
          isoCalendar.set(Calendar.YEAR, isoCalendar.get(Calendar.YEAR) + 1);
          isoCalendar.set(Calendar.ERA, GregorianCalendar.BC);
        } 
        //else {
        //  isoCalendar.set(Calendar.YEAR, year);
        //  isoCalendar.set(Calendar.ERA, GregorianCalendar.AD);
        //}
        
        return isoCalendar;
      } catch(ParseException e) {
        log.warn("Can't parse serialized calendar '" + parts[0] + "'", e);
      }
    }
    return null;
  }
  
  byte[] serialyze(Calendar date) {
    
    // String isoString = ISO8601.format(date);
    String calendarString = CALENDAR_FIELDS_SEPARATOR 
      + date.isLenient() + CALENDAR_FIELDS_DELIMITER 
      + date.getFirstDayOfWeek() + CALENDAR_FIELDS_DELIMITER 
      + date.getMinimalDaysInFirstWeek() + CALENDAR_FIELDS_DELIMITER 
      + date.getTimeZone().getID();
    // String sd = isoString + calendarString;
    String sd = DATETZ_FORMAT.format(date.getTime()) + calendarString;
    return sd.getBytes(); 
  }    
}
