/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.dataflow;

import java.util.ArrayList;
import java.util.List;

import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.impl.core.ItemChangeState;
import org.exoplatform.services.jcr.impl.core.ItemState;
import org.exoplatform.services.jcr.impl.core.SessionImpl;

/**
 * Created by The eXo Platform SARL        .
 *
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: ItemDataChangesLog.java 7657 2006-08-07 13:23:11Z peterit $
 * 
 * Stores collection of TransientItemData
 */
public class ItemDataChangesLog {
  
  protected final List <ItemState> items;
  protected String sessionId;
  
  // temporary!
  protected SessionImpl session;
  
  public ItemDataChangesLog(List <ItemState> items, SessionImpl session) {
    this.items = items;
    this.sessionId = null;
    this.session = session;
  }
  
  public ItemDataChangesLog(ItemState change, SessionImpl session ) {
    this(new ArrayList <ItemState> (), session);
    items.add(change);
  }
  
  public List <ItemState> getChanges() {
    return items;
  }
  
  public int getSize() {
    return items.size();
  }
  
  public ItemState findItemState(String itemUuid) {
    for (ItemState state: getChanges()) {
      if (state.getData().getUUID().equals(itemUuid)) {
        return state;
      }
    }
    return null;
  }
  
  public ItemState findItemState(InternalQPath itemPath) {
    for (ItemState state: getChanges()) {
      if (state.getData().getQPath().equals(itemPath)) {
        return state;
      }
    }
    return null;
  }
  
  public String getSessionId() {
    return sessionId;
  }
  
  public void setSessionId(String sessionId) {
    this.sessionId = sessionId;
  }

  public SessionImpl getSession() {
    return session;
  }
  
  public String dump() {
    String str = "ChangesLog: \n";
    for(int i=0; i<items.size(); i++)
      str+=" "+items.get(i).getData().getQPath().getAsString()+" "+ItemChangeState.getStateName(items.get(i).getState())+" \n";
    return str;
  }
}
