/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.dataflow;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.ValueFormatException;

import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.impl.core.value.ValueFactoryImpl;

/**
 * Created by The eXo Platform SARL
 * 15.05.2006
 * 
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: NodeDataReader.java 5800 2006-05-28 18:03:31Z geaz $
 */
public class NodeDataReader extends ItemDataReader {

  private HashMap<String, NodeInfo> nodes = new HashMap<String, NodeInfo>();
  private HashMap<String, NodeInfo> nodesByType = new HashMap<String, NodeInfo>();
  private PropertyDataReader nodePropertyReader = null;
  
  private List<NodeData> skiped = new ArrayList<NodeData>(); 
  private boolean rememberSkiped = false;
  
  private class NodeInfo {
    private InternalQName nodeName = null;
    private List<NodeDataReader> childNodesReaders = null;

    NodeInfo(InternalQName nodeName, List<NodeDataReader> childNodesReaders) {
      this.nodeName = nodeName;
      this.childNodesReaders = childNodesReaders;
    }
    
    public InternalQName getNodeName() {
      return nodeName;
    }

    public List<NodeDataReader> getChildNodesReaders() {
      return childNodesReaders;
    }
  }
  
  public NodeDataReader(NodeData node, DataManager dataManager, ValueFactoryImpl valueFactory) {
    super(node, dataManager, valueFactory);
  }
  
  public NodeDataReader forNodesByType(InternalQName name) {
    nodesByType.put(name.getAsString(), new NodeInfo(name, new ArrayList<NodeDataReader>()));
    return this;
  }
  
  public NodeDataReader forNode(InternalQName name) {
    nodes.put(name.getAsString(), new NodeInfo(name, new ArrayList<NodeDataReader>()));
    return this;
  }
  
  public List<NodeDataReader> getNodes(InternalQName name) throws PathNotFoundException {
    List<NodeDataReader> nr = nodes.get(name.getAsString()).getChildNodesReaders();
    if (nr.size()>0)
      return nr;
    throw new PathNotFoundException("Node with name " + parent.getQPath().getAsString() 
        + name.getAsString() + " not found");
  }
  
  public List<NodeDataReader> getNodesByType(InternalQName typeName) throws PathNotFoundException {
    List<NodeDataReader> nr = nodesByType.get(typeName.getAsString()).getChildNodesReaders();
    if (nr.size()>0)
      return nr;
    throw new PathNotFoundException("Nodes with type " + typeName.getAsString() + " not found. Parent "
        + parent.getQPath().getAsString());
  }
  
  public Value getPropertyValue(InternalQName name) throws ValueFormatException, PathNotFoundException {
    return nodePropertyReader.getPropertyValue(name);
  }
  
  public List<Value> getPropertyValues(InternalQName name) throws ValueFormatException, PathNotFoundException {
    return nodePropertyReader.getPropertyValues(name);
  }
  
  /**
   * Read node properties
   * @param name
   * @param type
   * @param multiValued
   * @return
   */
  public PropertyDataReader forProperty(InternalQName name, int type) {
    if (nodePropertyReader == null) {
      nodePropertyReader = new PropertyDataReader(parent, dataManager, valueFactory);
    } 
    return nodePropertyReader.forProperty(name, type);
  }
  
  public void read() throws RepositoryException {

    cleanReaders();

    if (nodePropertyReader != null) {
      nodePropertyReader.read();
    }

    if (nodes.size() > 0 || nodesByType.size() > 0 || rememberSkiped) {
      
      List<NodeData> ndNodes = dataManager.getChildNodesData(parent);
      for (NodeData node : ndNodes) {

        boolean isSkiped = true;

        NodeDataReader cnReader = new NodeDataReader(node, dataManager, valueFactory);
        // cnReader.read(); // lazy now

        NodeInfo nodeInfo = nodes.get(node.getQPath().getName().getAsString());
        if (nodeInfo != null) {
          nodeInfo.getChildNodesReaders().add(cnReader);
          isSkiped = false;
        }

        NodeInfo nodesByTypeInfo = nodesByType.get(node.getPrimaryTypeName().getAsString());
        if (nodesByTypeInfo != null) {
          nodesByTypeInfo.getChildNodesReaders().add(cnReader);
          isSkiped = false;
        }

        if (isSkiped && rememberSkiped) {
          skiped.add(node);
        }
      }
    }
  }
  
  private void cleanReaders() {
    for (Map.Entry<String,NodeInfo> nodesEntry: nodes.entrySet()) {
      nodesEntry.getValue().getChildNodesReaders().clear();
    }
    for (Map.Entry<String,NodeInfo> nodesEntry: nodesByType.entrySet()) {
      nodesEntry.getValue().getChildNodesReaders().clear();
    }
    skiped.clear();
  }
  
  public void clean() {
    nodes = new HashMap<String, NodeInfo>();
    nodesByType = new HashMap<String, NodeInfo>();
    nodePropertyReader = null;
    skiped = new ArrayList<NodeData>();
  }

  public boolean isRememberSkiped() {
    return rememberSkiped;
  }

  public void setRememberSkiped(boolean rememberSkiped) {
    this.rememberSkiped = rememberSkiped;
  }

  public List<NodeData> getSkiped() {
    return skiped;
  }
  
}
