/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.dataflow.session;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import javax.jcr.AccessDeniedException;
import javax.jcr.InvalidItemStateException;
import javax.jcr.ReferentialIntegrityException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.lock.LockException;
import javax.jcr.nodetype.ConstraintViolationException;

import org.apache.commons.logging.Log;
import org.exoplatform.services.jcr.access.PermissionType;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.impl.core.CredentialsImpl;
import org.exoplatform.services.jcr.impl.core.ItemChangeState;
import org.exoplatform.services.jcr.impl.core.ItemImpl;
import org.exoplatform.services.jcr.impl.core.ItemState;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.PropertyImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.core.version.VersionHistoryInitializer;
import org.exoplatform.services.jcr.impl.dataflow.DataManager;
import org.exoplatform.services.jcr.impl.dataflow.ItemDataChangesLog;
import org.exoplatform.services.jcr.impl.dataflow.TransientItemData;
import org.exoplatform.services.jcr.impl.dataflow.TransientNodeData;
import org.exoplatform.services.jcr.impl.dataflow.TransientPropertyData;
import org.exoplatform.services.jcr.impl.dataflow.persistent.BatcheableWorkspaceDataManager;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.transaction.TransactionException;
import org.exoplatform.services.transaction.TransactionResource;

/**
 * Created by The eXo Platform SARL        .
 *
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: TransactionableDataManager.java 7757 2006-08-08 15:40:14Z peterit $
 */
public class TransactionableDataManager implements DataManager, TransactionResource {

  private BatcheableWorkspaceDataManager storageDataManager;
  
  private SessionImpl session;
  
  protected static Log log = ExoLogger.getLogger("jcr.TransactionableDataManager");
  
  private SessionChangesLog transactionLog;
  
  private VersionHistoryInitializer versionHistoryInitializer = null;
  
  protected String infoString = null;
  private SimpleDateFormat infoDateFormater = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSSS");  
  
  public TransactionableDataManager(BatcheableWorkspaceDataManager storageDataManager,
      SessionImpl session) {
    super();
    this.storageDataManager = storageDataManager;
    this.session = session;
    this.versionHistoryInitializer = new VersionHistoryInitializer(storageDataManager, session);
    
    try {
      infoString = session.getSessionInfo()  
        + ", " + infoDateFormater.format(storageDataManager.getCurrentTime().getTime())
        + ", " + session;
    } catch(Exception e) {
      log.warn("TransactionableDataManager error of read session info: " + e.getMessage());
      infoString = "[Error of read session info: " + e.getMessage() + "]";
    }
  }
  
  @Override
  public String toString() {
    return super.toString() + " [" + infoString + "]";
  }  

  // --------------- ItemDataConsumer --------

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.dataflow.ItemDataConsumer#getChildNodesData(org.exoplatform.services.jcr.datamodel.NodeData)
   */
  public List<NodeData> getChildNodesData(NodeData parent) throws RepositoryException {
    List <NodeData> nodes = storageDataManager.getChildNodesData(parent);

    // merge data
    if(txStarted()) {
      for(ItemState state:transactionLog.getChildStates(parent.getQPath(), true)) {
        nodes.remove(state.getData());
        if(state.getState() != ItemChangeState.DELETED)
          nodes.add((NodeData)state.getData());
      }
    }
    return nodes; 
  }


  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.dataflow.ItemDataConsumer#getChildPropertiesData(org.exoplatform.services.jcr.datamodel.NodeData)
   */
  public List<PropertyData> getChildPropertiesData(NodeData parent) throws RepositoryException {
    List <PropertyData> props = storageDataManager.getChildPropertiesData(parent);

    // merge data
    if(txStarted()) {
      for(ItemState state:transactionLog.getChildStates(parent.getQPath(), false)) {
        props.remove(state.getData());
        if(state.getState() != ItemChangeState.DELETED)
          props.add((PropertyData)state.getData());
      }
    }
    return props; 
  }


  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.dataflow.ItemDataConsumer#getItemData(org.exoplatform.services.jcr.datamodel.InternalQPath)
   */
  public ItemData getItemData(InternalQPath path) throws RepositoryException {
    ItemData data = null;
    if(txStarted()) {
      ItemState state = transactionLog.getState(path);
      if(state != null)
        data = state.getData();
    }
    if(data != null)
      return data;
    else
      return storageDataManager.getItemData(path);
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.dataflow.ItemDataConsumer#getItemData(java.lang.String)
   */
  public ItemData getItemData(String uuid) throws RepositoryException {
    ItemData data = null;
    if(txStarted()) {
      ItemState state = transactionLog.getState(uuid);
      if(state != null)
        data = state.getData();
    }
    if(data != null)
      return data;
    else
      return storageDataManager.getItemData(uuid);
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.dataflow.ItemDataConsumer#getReferencesData(java.lang.String)
   */
  public List<PropertyData> getReferencesData(String uuid) throws RepositoryException {
    return storageDataManager.getReferencesData(uuid);
  }
  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.dataflow.ItemDataConsumer#getChildNodesCount(org.exoplatform.services.jcr.datamodel.NodeData)
   */
  public int getChildNodesCount(NodeData parent) throws RepositoryException {
    if(txStarted()) {
      return getChildNodesData(parent).size();
    } else
      return storageDataManager.getChildNodesCount(parent);
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.dataflow.ItemDataConsumer#getChildPropertiesCount(org.exoplatform.services.jcr.datamodel.NodeData)
   */
  public int getChildPropertiesCount(NodeData parent) throws RepositoryException {
    if(txStarted()) {
      return getChildPropertiesData(parent).size();
    } else
      return storageDataManager.getChildPropertiesCount(parent);
  }
    
  // ---------------  --------

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.dataflow.TransactionResource#start(org.exoplatform.services.jcr.impl.dataflow.TransactionLifecycleManager)
   */
  public void start() {
    log.debug("tx start() "+this+" txStarted(): "+txStarted());
    if(!txStarted())
      transactionLog = new SessionChangesLog(session);
  }
  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.dataflow.TransactionResource#commit(org.exoplatform.services.jcr.impl.dataflow.TransactionLifecycleManager)
   */
  public void commit() throws TransactionException {
    if(txStarted()) {
      log.debug("tx commit() " + this + "\n"+transactionLog.dump());
      try {
        storageDataManager.save(transactionLog);
        transactionLog = null;
      } catch (InvalidItemStateException e) {
        throw new TransactionException(e);
      } catch (RepositoryException e) {
        throw new TransactionException(e);
      }
    }
  }
 
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.dataflow.TransactionResource#rollback(org.exoplatform.services.jcr.impl.dataflow.TransactionLifecycleManager)
   */
  public void rollback() {
    log.debug("tx rollback() "+this+ (transactionLog != null ? "\n"+transactionLog.dump() : "[NULL]"));
    if(txStarted()) 
      transactionLog = null;
  }

  /**
   * Updates the manager with new changes
   * If transaction is started it will fill manager's changes log,
   * else just move changes to workspace storage manager
   * It saves the changes AS IS - i.e. id DOES NOT care about 
   * cloning of this objects etc. 
   * @param changes
   * @throws RepositoryException
   */
  public void save(ItemDataChangesLog changes) throws RepositoryException  {
    
    if (log.isDebugEnabled())
      log.debug("save() "+this+" txStarted: " + txStarted() 
          + "\n====== Changes ======\n" + (changes != null ? "\n"+changes.dump() : "[NULL]") + "====================="
          );
    
    versionHistoryInitializer.saveVersionable(changes);
    
    if(txStarted())
      transactionLog.addAll(changes.getChanges());
    else
      storageDataManager.save(changes);
  }
  
  public boolean txStarted() {
    return transactionLog != null;
  }
  
  public boolean txHasPendingChages() {
    return txStarted() && transactionLog.getSize()>0;
  }
  
  /**
   * Temporary method, will be removed after Session refactoring
   * It MAKES copy of inputting data 
   * For newly ADDED nodes it will also traverse children and add
   * them to output(save or pass) 
   * @param item - an item
   * @param validate - flag indicated that item-related validation will be performed
   */
   public void saveItem(ItemImpl item, boolean validate) throws RepositoryException {

     if (validate)
       validate(item);

     ItemDataChangesLog changeLog;
     if (item.isNode() && item.isNew()) { // Traverse ADDED nodes and pass to
       // add new node
       List<ItemState> changes = new ArrayList<ItemState>();
       traverseAddedNode(item.getActualItemData(), changes);
       changeLog = new ItemDataChangesLog(changes, session);
     } else {
       // add property, update, delete items
       TransientItemData newData = copyItemData(item.getActualItemData());
       ItemState state = new ItemState(newData, item.getState());
       changeLog = new ItemDataChangesLog(state, session);
       //log.debug("save: " + state.getData().getQPath().getAsString() + " "
       //   + ItemChangeState.getStateName(state.getState()));
     }
     save(changeLog);
  }
   
 /**
  * Temporary method, will be removed after Session refactoring
  * It MAKES copy of inputting data 
  * For newly ADDED nodes it will also traverse children and add
  * them to output(save or pass) 
  * @param item
  */
  public void saveItem(ItemImpl item) throws RepositoryException {
      saveItem(item, true);
  } 

  protected void traverseAddedNode(TransientItemData item, List<ItemState> items)
      throws RepositoryException {
 
    items.add(new ItemState(copyItemData(item), ItemChangeState.ADDED));

    // traverse if node
    if (item.isNode()) {
      TransientNodeData node = (TransientNodeData)item;
      List props = node.getChildProperties();
      for (int i = 0; i < props.size(); i++) {
        traverseAddedNode((TransientPropertyData) props.get(i), items);
      }
      List nodes = node.getChildNodes();
      for (int i = 0; i < nodes.size(); i++) {
        traverseAddedNode((TransientNodeData) nodes.get(i), items);
      }
    } 
  }

  private TransientItemData copyItemData(TransientItemData item) {
    // make a copy
    if (item.isNode()) {

      TransientNodeData node = (TransientNodeData)item;
      return new TransientNodeData(node.getQPath(), node.getUUID(), 
        node.getPersistedVersion(), node.getPrimaryTypeName(), node.getMixinTypeNames(),
        node.getOrderNumber(), node.getParentUUID(), node.getOrderDirection());
    } else {
      
      TransientPropertyData prop = (TransientPropertyData)item;
      // make a copy
      TransientPropertyData newData = new TransientPropertyData(
          prop.getQPath(), prop.getUUID(), prop.getPersistedVersion(),
          prop.getType(), prop.getParentUUID(), prop.isMultiValued());
      newData.setValues(prop.getValues());
      return newData;
    }
  }
  
  private void validate(ItemImpl item) throws RepositoryException,
      AccessDeniedException, ReferentialIntegrityException, LockException,
      ConstraintViolationException {

    // Check permissions
    // delete item
    if (item.getState() == ItemChangeState.DELETED) {

      // check for manadatory property
      if (!item.isNode()) {

        PropertyImpl prop = (PropertyImpl) item;
        if (prop.getDefinition().isMandatory()) {
          NodeImpl itemParent = (NodeImpl) item.getParent();
          // Check if parent will be removed
          if (itemParent.isRoot()
              || itemParent.getState() != ItemChangeState.DELETED) {
            throw new ConstraintViolationException(
                "Can not remove mandatory property " + prop.getPath());
          } // else perent node will be removed too
        }
      }

      if (!item.hasPermission(PermissionType.REMOVE))
        throw new AccessDeniedException("Access denied: REMOVE "
            + item.getPath()+" for: "+session.getUserID());

    }

    // add node
    if (item.isNode() && item.getState() == ItemChangeState.ADDED) {

      // check for manadatory property
      if (item.isNode()) {
        NodeImpl node = (NodeImpl) item;
        node.validateMandatoryChildren();
      }

      if (!item.hasPermission(PermissionType.ADD_NODE))
        throw new AccessDeniedException("Access denied: ADD_NODE "
            + item.getPath()+" for: "+session.getUserID());
    }

    // add/update property
    if (!item.isNode()
        && (item.getState() == ItemChangeState.UPDATED || item.getState() == ItemChangeState.ADDED)) {

      if (!item.hasPermission(PermissionType.SET_PROPERTY))
        throw new AccessDeniedException("Access denied: SET_PROPERTY "
            + item.getPath()+" for: "+session.getUserID());

    }
  }
}
