/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.storage.jdbc;

import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;

import javax.jcr.RepositoryException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.exoplatform.services.database.jdbc.DBSchemaCreator;
import org.exoplatform.services.jcr.config.RepositoryConfigurationException;
import org.exoplatform.services.jcr.config.WorkspaceEntry;
import org.exoplatform.services.jcr.impl.storage.WorkspaceDataContainerBase;
import org.exoplatform.services.jcr.impl.storage.jdbc.update.StorageUpdateManager;
import org.exoplatform.services.jcr.impl.storage.value.ValueStoragePluginProvider;
import org.exoplatform.services.jcr.storage.WorkspaceStorageConnection;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: JDBCWorkspaceDataContainer.java 9155 2006-09-27 09:02:21Z peterit $
 */
public class JDBCWorkspaceDataContainer extends WorkspaceDataContainerBase {

  protected static Log log = ExoLogger.getLogger("jcr.JDBCWorkspaceDataContainer");
  
  protected String containerName;
  protected String sourceName;
  protected boolean multiDB;
  private DataSource ds;
  private ValueStoragePluginProvider valueStorageProvider;
  private String storageVersion;

  /**
   * Constructor with no value storage plugins configured
   * @param wsConfig
   * @param dbCreator
   * @throws RepositoryConfigurationException
   * @throws NamingException
   */
  public JDBCWorkspaceDataContainer(WorkspaceEntry wsConfig, 
      DBSchemaCreator dbCreator) 
  throws RepositoryConfigurationException, NamingException, RepositoryException, IOException {
    this(wsConfig, dbCreator, null);
  }
  
  /**
   * Constructor with value storage plugins
   * @param wsConfig
   * @param dbCreator
   * @param valueStrorageProvider
   * @throws RepositoryConfigurationException
   * @throws NamingException
   */
  public JDBCWorkspaceDataContainer(WorkspaceEntry wsConfig, 
      DBSchemaCreator dbCreator,
      ValueStoragePluginProvider valueStorageProvider) 
  throws RepositoryConfigurationException, NamingException, RepositoryException, IOException {
    
    this.containerName = wsConfig.getName();
    this.sourceName = wsConfig.getContainer().getParameterValue("sourceName");
    this.valueStorageProvider = valueStorageProvider;
    
    multiDB = Boolean.parseBoolean(
        wsConfig.getContainer().getParameterValue("multi-db"));
    
    Context context = new InitialContext();
    ds = (DataSource) context.lookup(sourceName);

    String suParam = null;
    boolean enableStorageUpdate = false;
    try {
      suParam = wsConfig.getContainer().getParameterValue("update-storage");
      enableStorageUpdate = Boolean.parseBoolean(suParam);
    } catch (RepositoryConfigurationException e) {
      log.debug("update-storage parameter is not set "+sourceName);
    }
    checkVersion(enableStorageUpdate);

    log.info("JDBCWorkspaceDataContainer (cEntry): " + containerName + ", dataSource: " + sourceName);
  }
  
  public String checkVersion(boolean enableStorageUpdate) throws RepositoryException {
    this.storageVersion = StorageUpdateManager.checkVersion(sourceName, ds, multiDB, enableStorageUpdate);
    
    return this.storageVersion;
  }
    
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.storage.WorkspaceDataContainer#openConnection()
   */
  public WorkspaceStorageConnection openConnection() throws RepositoryException {
    
    try {

      Connection conn = ds.getConnection();
      
      // Fix for Sybase jConnect JDBC driver bug. 
      // Which throws SQLException(JZ016: The AutoCommit option is already set to false)
      // if conn.setAutoCommit(false) called twise or more times with value 'false'.
      if (conn.getAutoCommit()) {
        conn.setAutoCommit(false);
      }
      
      if (multiDB) 
        return new MultiDbJDBCConnection(conn, containerName, valueStorageProvider);
      else 
        return new SingleDbJDBCConnection(conn, containerName, valueStorageProvider);

    } catch (SQLException e) {
      throw new RepositoryException(e);
    }
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.storage.WorkspaceDataContainer#getName()
   */
  public String getName() {
    return containerName;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.storage.WorkspaceDataContainer#getInfo()
   */
  public String getInfo() {
    String str = "Info: JDBC based JCR Workspace Data container \n";
    return str;
  }

  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.storage.DataContainer#getStorageVersion()
   */
  public String getStorageVersion() {
    return storageVersion;
  }

}