/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.xml;

import java.util.List;

import javax.jcr.RepositoryException;
import javax.jcr.nodetype.PropertyDefinition;

import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.RepositoryImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeManagerImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.PropertyDefinitionImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.PropertyDefinitions;
import org.xml.sax.ContentHandler;

/**
 * Created by The eXo Platform SARL
 * Author : Peter Nedonosko
 *          peter.nedonosko@exoplatform.com.ua
 * 22.09.2006
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: ImporteBase.java 9060 2006-09-22 22:11:39Z peterit $
 */
abstract public class ImporteBase implements ContentHandler {

  protected int uuidBehavior = -1;
    
  protected SessionImpl       session;

  protected NodeImpl          rootNode;
  
  ImporteBase(NodeImpl parent, int uuidBehavior) {
  
    this.session = (SessionImpl) parent.getSession();
    this.uuidBehavior = uuidBehavior;
  }
  
  protected NodeTypeImpl findNodeType(String ntName) throws RepositoryException {
    
    NodeTypeManagerImpl ntMan = (NodeTypeManagerImpl) ((RepositoryImpl) session.getRepository()).getNodeTypeManager();
    return (NodeTypeImpl) ntMan.getNodeType(ntName);
  }
  
  protected NodeTypeImpl findNodeType(InternalQName ntName) throws RepositoryException {
    
    NodeTypeManagerImpl ntMan = (NodeTypeManagerImpl) ((RepositoryImpl) session.getRepository()).getNodeTypeManager();
    return (NodeTypeImpl) ntMan.getNodeType(ntName);
  }
  
  protected PropertyDefinition getPropertyDefinition(String propertyName, List<NodeTypeImpl> nodeTypes) {
    PropertyDefinition pdResidual = null;
    for (NodeTypeImpl nt: nodeTypes){
      PropertyDefinitions pds = nt.getPropertyDefinitions(propertyName);
      PropertyDefinition pd = pds.getAnyDefinition();
      if ( pd != null) {
        if (((PropertyDefinitionImpl)pd).isResidualSet()) {
          pdResidual = pd;
        } else {
          return pd;
        }
      }
    }
    return pdResidual;
  }
  
  protected boolean isReferenceable(List<NodeTypeImpl> nodeTypes) {
    
    for (NodeTypeImpl nt: nodeTypes){
      if (nt.isNodeType(Constants.MIX_REFERENCEABLE)) {
        return true;
      }
    }
    return false;
  }
  
}
