/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.api.blob;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Calendar;

import java.net.URL;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.query.Query;
import javax.jcr.query.QueryResult;

import org.exoplatform.services.jcr.JcrAPIBaseTest;
import org.exoplatform.services.jcr.impl.core.value.BinaryValue;

/**
 * Created by The eXo Platform SARL.
 */

public class TestBinaryValue extends JcrAPIBaseTest {
  //Make sure you have the files pointed below!	
  private Node testBinaryValue = null;	
  private static int FILES_COUNT = 2;
  private static String LOCAL_BIG_FILE = "src/test/resources/BigFile.zip";
  private static String LOCAL_SMALL_FILE = "src/test/resources/SmallFile.zip";
  private static String REMOTE_BIG_FILE = "\\\\Exooffice\\public\\Tmp\\resources\\BigFile.zip";
  private static String REMOTE_SMALL_FILE = "\\\\Exooffice\\public\\Tmp\\resources\\SmallFile.zip";
  private static String URL_BIG_FILE = "http://localhost:8080/ecm/jcr?workspace=production&path=/testBinaryValue/BigFile.zip";
  private static String URL_SMALL_FILE = "http://localhost:8080/ecm/jcr?workspace=production&path=/testBinaryValue/SmallFile.zip";
  
  public void setUp() throws Exception {
	super.setUp();  
	testBinaryValue = root.addNode("testBinaryValue");
	session.save();
  }
  
  public void testLocalBigFiles() throws Exception {
    Node testLocalBigFiles = testBinaryValue.addNode("testLocalBigFiles");
    long startTime, endTime;
    startTime = System.currentTimeMillis(); // to get the time of start
    for (int i = 0; i < FILES_COUNT; i++) {
      Node localBigFile = testLocalBigFiles.addNode("bigFile" + i, "nt:file");
      Node contentNode = localBigFile.addNode("jcr:content", "nt:resource");
      contentNode.setProperty("jcr:encoding", "UTF-8");
      contentNode.setProperty("jcr:data", new FileInputStream(LOCAL_BIG_FILE));
      contentNode.setProperty("jcr:mimeType", "application/octet-stream "); 
      contentNode.setProperty("jcr:lastModified", Calendar.getInstance());
    }
    session.save();
    endTime = System.currentTimeMillis();    
    log.info("Execution time after adding and saving (local big):" + ((endTime - startTime) / 1000) + "ms");
  }

  public void testLocalSmallFiles() throws Exception {
    Node testLocalSmallFiles = testBinaryValue.addNode("testLocalSmallFiles");
    long startTime, endTime;
    startTime = System.currentTimeMillis(); // to get the time of start
    for (int i = 0; i < FILES_COUNT; i++) {
      Node localSmallFile = testLocalSmallFiles.addNode("smallFile" + i, "nt:file");
      Node contentNode = localSmallFile.addNode("jcr:content", "nt:resource");
      contentNode.setProperty("jcr:encoding", "UTF-8");
      contentNode.setProperty("jcr:data", new FileInputStream(LOCAL_SMALL_FILE));
      contentNode.setProperty("jcr:mimeType", "application/octet-stream "); 
      contentNode.setProperty("jcr:lastModified", Calendar.getInstance());
    }
    session.save();
    endTime = System.currentTimeMillis();    
    log.info("Execution time after adding and saving: (local small)" + ((endTime - startTime) / 1000) + "ms");
  }  

  public void testRemoteBigFiles() throws Exception {
    Node testRemoteBigFiles = testBinaryValue.addNode("testRemoteBigFiles");
    long startTime, endTime;
    startTime = System.currentTimeMillis(); // to get the time of start
    for (int i = 0; i < FILES_COUNT; i++) {
      Node remoteBigFile = testRemoteBigFiles.addNode("bigFile" + i, "nt:file");
      Node contentNode = remoteBigFile.addNode("jcr:content", "nt:resource");
      contentNode.setProperty("jcr:encoding", "UTF-8");
      contentNode.setProperty("jcr:data", new FileInputStream(REMOTE_BIG_FILE));
      contentNode.setProperty("jcr:mimeType", "application/octet-stream "); 
      contentNode.setProperty("jcr:lastModified", Calendar.getInstance());
    }
    session.save();
    endTime = System.currentTimeMillis();    
    log.info("Execution time after adding and saving (remote big):" + ((endTime - startTime) / 1000) + "ms");
  }
  
  public void testRemoteSmallFiles() throws Exception {
    Node testRemoteSmallFiles = testBinaryValue.addNode("testRemoteSmallFiles");
    long startTime, endTime;
    startTime = System.currentTimeMillis(); // to get the time of start
    for (int i = 0; i < FILES_COUNT; i++) {
      Node remoteSmallFile = testRemoteSmallFiles.addNode("smallFile" + i, "nt:file");
      Node contentNode = remoteSmallFile.addNode("jcr:content", "nt:resource");
      contentNode.setProperty("jcr:encoding", "UTF-8");
      contentNode.setProperty("jcr:data", new FileInputStream(REMOTE_SMALL_FILE));
      contentNode.setProperty("jcr:mimeType", "application/octet-stream "); 
      contentNode.setProperty("jcr:lastModified", Calendar.getInstance());
    }
    session.save();
    endTime = System.currentTimeMillis();    
    log.info("Execution time after adding and saving: (remote small)" + ((endTime - startTime) / 1000) + "ms");
  }

  public void testUrlBigFiles() throws Exception {
    Node testUrlBigFiles = testBinaryValue.addNode("testUrlBigFiles");
    long startTime, endTime;
    startTime = System.currentTimeMillis(); // to get the time of start
    for (int i = 0; i < FILES_COUNT; i++) {
      Node urlBigFile = testUrlBigFiles.addNode("bigFile" + i, "nt:file");
      Node contentNode = urlBigFile.addNode("jcr:content", "nt:resource");
      contentNode.setProperty("jcr:encoding", "UTF-8");
      contentNode.setProperty("jcr:data", new URL(URL_BIG_FILE).openStream());
      contentNode.setProperty("jcr:mimeType", "application/octet-stream "); 
      contentNode.setProperty("jcr:lastModified", Calendar.getInstance());
    }
    session.save();
    endTime = System.currentTimeMillis();    
    log.info("Execution time after adding and saving: (url big)" + ((endTime - startTime) / 1000) + "ms");
  }  
  
  public void testUrlSmallFiles() throws Exception {
    Node testUrlSmallFiles = testBinaryValue.addNode("testUrlSmallFiles");
    long startTime, endTime;
    startTime = System.currentTimeMillis(); // to get the time of start
    for (int i = 0; i < FILES_COUNT; i++) {
      Node urlSmallFile = testUrlSmallFiles.addNode("smallFile" + i, "nt:file");
      Node contentNode = urlSmallFile.addNode("jcr:content", "nt:resource");
      contentNode.setProperty("jcr:encoding", "UTF-8");
      contentNode.setProperty("jcr:data", new URL(URL_SMALL_FILE).openStream());
      contentNode.setProperty("jcr:mimeType", "application/octet-stream "); 
      contentNode.setProperty("jcr:lastModified", Calendar.getInstance());
    }
    session.save();
    endTime = System.currentTimeMillis();    
    log.info("Execution time after adding and saving: (url small)" + ((endTime - startTime) / 1000) + "ms");
  }  
  
  protected void tearDown() throws Exception {
    root.getNode("testBinaryValue").remove();
    root.save();
    super.tearDown();
  }  
}
