/**
 **************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.rmi;

import java.net.MalformedURLException;

import java.io.*;
import java.rmi.*;
import javax.jcr.*;
import javax.naming.*;
import org.apache.jackrabbit.rmi.client.*;
import org.apache.jackrabbit.rmi.remote.*;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:lautarul@gmail.com">Roman Pedchenko</a>
 * @version $Id: RMIRepositoryWrapper.java 3846 2006-01-18 14:08:07Z lautarul $
 */

public class RMIRepositoryWrapper implements Serializable, Referenceable, Repository {

  static final String URL_ADDRTYPE = "url";

  private String url;

  private transient Repository delegatee = null;

  public RMIRepositoryWrapper(String url) throws ClassCastException, NotBoundException,
      MalformedURLException, RemoteException {
    this.url = url;
    init();
  }

  public RMIRepositoryWrapper(String url, Repository repository) {
    this.url = url;
    delegatee = repository;
  }

  private void init() throws ClassCastException, MalformedURLException, NotBoundException, RemoteException {
    RemoteRepository remote = (RemoteRepository) Naming.lookup(url);
    delegatee = (Repository) (new ClientAdapterFactory()).getRepository(remote);
  }
  
  public String[] getDescriptorKeys() {
    return delegatee.getDescriptorKeys();
  }

  public String getDescriptor(String key) {
    return delegatee.getDescriptor(key);
  }

  public Session login(Credentials credentials) throws LoginException,
      NoSuchWorkspaceException, RepositoryException {
    return delegatee.login(credentials);
  }

  public Session login(String workspaceName) throws LoginException,
      NoSuchWorkspaceException, RepositoryException {
    return delegatee.login(workspaceName);
  }

  public Session login() throws LoginException, NoSuchWorkspaceException,
      RepositoryException {
    return delegatee.login();
  }

  public Session login(Credentials credentials, String workspaceName)
      throws LoginException, NoSuchWorkspaceException, RepositoryException {
    return delegatee.login(credentials, workspaceName);
  }
  private void writeObject(ObjectOutputStream out) throws IOException {
    // delegate to default implementation
    out.defaultWriteObject();
  }

  private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException, NotBoundException {
    // delegate deserialization to default implementation
    in.defaultReadObject();
    // initialize reconstructed instance
    init();
  }

  public Reference getReference() throws NamingException {
    Reference ref = new Reference(
      RMIRepositoryWrapper.class.getName(),
      RepositoryFactory.class.getName(),
      null);
    ref.add(new StringRefAddr(URL_ADDRTYPE, url));
    return ref;
  }
}
