package org.exoplatform.services.webdav.impl;

import javax.servlet.http.HttpServletRequest;

import org.apache.jackrabbit.webdav.DavLocatorFactory;
import org.apache.jackrabbit.webdav.DavResourceLocator;
import org.apache.jackrabbit.webdav.WebdavRequestImpl;

public class ExoWebdavRequestImpl extends WebdavRequestImpl {
    
    private String workspaceName = "";
    private HttpServletRequest httpRequest = null;
    private DavLocatorFactory locatorFactory = null;
    private String exoPrefix = "";
    
    public ExoWebdavRequestImpl(HttpServletRequest httpRequest, DavLocatorFactory factory, String workSpace) {
        super(httpRequest, factory);
        this.httpRequest = httpRequest;
        this.locatorFactory = factory;
        workspaceName = workSpace;        
        exoPrefix = httpRequest.getScheme() + "://" + httpRequest.getHeader("Host") + httpRequest.getContextPath();
    }

    public DavResourceLocator getRequestLocator() {
        String absPath = exoPrefix + getServletPath() + "/" + workspaceName;        
        String requestURI = getRequestURI();
        String sourcePath = requestURI.substring((httpRequest.getContextPath() + httpRequest.getServletPath() + "/" + workspaceName).length());
        if (sourcePath.equals("")) {
            sourcePath = "/";
        } else {
            if (sourcePath.length() > 1) {
                if (sourcePath.charAt(sourcePath.length() - 1) == '/') {
                    sourcePath = sourcePath.substring(0, sourcePath.length() - 1);
                }                    
            }            
        }
        return locatorFactory.createResourceLocator(absPath, "", sourcePath);
    }
    
    public DavResourceLocator getDestinationLocator() {
        String destination = httpRequest.getHeader(HEADER_DESTINATION);
        String absPath = exoPrefix + getServletPath() + "/" + workspaceName;
        String sourcePath = "/";        
        if (destination != null) {
            if (destination.startsWith(absPath)) {
                sourcePath = destination.substring(absPath.length());                
            }
            if (sourcePath.length() > 1) {
                if (sourcePath.charAt(sourcePath.length() - 1) == '/') {
                    sourcePath = sourcePath.substring(0, sourcePath.length() - 1);
                }
            }
        }                
        return locatorFactory.createResourceLocator(absPath, "", sourcePath);        
    }

    public String GetWorkspaceName() {
        return workspaceName;
    }
    
}
